/*****************************************************************************/
/*****************************************************************************/
/**  Analizador Lexico para MenosC.09                                       **/
/**                                                                         **/
/**                     Jose Miguel Benedi, 2009-2010 <jbenedi@dsic.upv.es> **/
/*****************************************************************************/
/* V. EVR 10                                                                */
/*****************************************************************************/
%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "header.h"
#include "asin.h"
#define  Devolver(x) {if (verbosidad) ECHO; return(x);}
/*****************************************************************************/
/*     Perfil de las operaciones definidas en la seccin de codigo           */
/*****************************************************************************/
void crea_nombre();
void crea_ctentera();
void trunc_ctreal();
/*****************************************************************************/
/*          Definicion de Expresiones Regulares auxiliares                   */
/*****************************************************************************/
%}
%option yylineno
%option caseless
%option noyywrap

delimitador	[ \t]+
comentario      "//"(.)*"\n"

letra 		[a-zA-Z]
digito		[0-9]
nentero		{digito}+
nreal		{nentero}"."{digito}*
identificador	{letra}({letra}|{digito})*
%{
/*****************************************************************************/
/*  Definicion de las Expresiones Regulares que determinan los patrones      */
/*  asociados con los simbolos del lenguaje.                                 */
/*****************************************************************************/
%}
%%	
{delimitador}   { if (verbosidad) ECHO;}
{comentario}    { if (verbosidad) {ECHO; printf("%3d.- ", yylineno);}}
"\n"            { if (verbosidad) {ECHO; printf("%3d.- ", yylineno);}}

else            { Devolver(ELSE_);  }
if              { Devolver(IF_);    }
int             { Devolver(INT_);   }
print           { Devolver(PRINT_); }
read            { Devolver(READ_);  }
return          { Devolver(RETURN_);}
while           { Devolver(WHILE_); }

"!="            { yylval.ent = DISTINTO; Devolver(OPREL_); }
"=="            { yylval.ent = IGUAL;    Devolver(OPREL_); }
">"             { yylval.ent = MAYOR;    Devolver(OPREL_); }
"<"             { yylval.ent = MENOR;    Devolver(OPREL_); }
">="            { yylval.ent = MAYORIG;  Devolver(OPREL_); }
"<="            { yylval.ent = MENORIG;  Devolver(OPREL_); }

"*"             { yylval.ent = EMULT;   Devolver(OPMUL_); }
"/"             { yylval.ent = EDIVI;   Devolver(OPMUL_); }

"+"             { yylval.ent = ESUM;   Devolver(OPADD_); }
"-"             { yylval.ent = EDIF; Devolver(OPADD_); }

"="             { yylval.ent = ASIG;       Devolver(OPASIG_);   }
"+="            { yylval.ent = MASASIG;    Devolver(OPASIG_);   }
"-="            { yylval.ent = MENOSASIG;  Devolver(OPASIG_);   }

"++"            { yylval.ent = ESUM;      Devolver(OPINCDEC_); }
"--"            { yylval.ent = EDIF; Devolver(OPINCDEC_); }

"("             { Devolver(APAR_);     }
")"             { Devolver(CPAR_);     }
"["             { Devolver(ACOR_);     }
"]"             { Devolver(CCOR_);     }
"{"             { Devolver(ALLA_);     }
"}"             { Devolver(CLLA_);     }
","             { Devolver(COMA_);     }
";"             { Devolver(PUNTOCOMA_);}

{nentero}       { crea_ctentera(); Devolver(CTE_);}
{nreal}	        { trunc_ctreal();  Devolver(CTE_);}
{identificador} { crea_nombre();   Devolver(ID_); }

.		{ if (verbosidad) {printf("<<"); ECHO; printf(">>");}
                  yyerror("ERROR: caracter desconocido.");
                }
%%
/*****************************************************************************/
/*****************************************************************************/
void crea_nombre()
/* Comprueba que el identificador no exceda la talla maxima (14); en ese     */
/* caso lo trunca.                                                           */
{
  char *t;
  if (yyleng > 14) { 
    printf("\n======> ATENCION: identificador truncado a 14 caracteres.\n");  
    yyleng = 14;
    }
  t = (char *)malloc(yyleng+1); strncpy(t, yytext, yyleng);
  t[yyleng] = 0; yylval.ident = t;
}
/*****************************************************************************/
void crea_ctentera()
/* Transforma una subcadena a la constante entera que representa             */
{
  yylval.ent = atoi(yytext);
}
/*****************************************************************************/
void trunc_ctreal()
/* Transforma una subcadena, que representa una constante real, a una        */
/* constante entera truncandola.                                             */
{   
    char *t;
    printf("\n======> ATENCION: constante real truncada a entera.\n");  
    yyleng = strcspn(yytext,".");
    t = (char *)malloc(yyleng+1); strncpy(t, yytext, yyleng);
    t[yyleng] = 0; yylval.ent = atoi(yytext);
}
/*****************************************************************************/
/*****************************************************************************/
