%{
/*****************************************************************************/
/*****************************************************************************/
/**  Analizador Sintactico-Semantico para MenosC.09                         **/
/**                                                                         **/
/**                     Jose Miguel Benedi, 2009-2010 <jbenedi@dsic.upv.es> **/
/*****************************************************************************/
/* V. EVR 10                                                                */
/*****************************************************************************/
#include <stdio.h>
#include "header.h"
%}
/********************** Tipos de datos para las declaraciones %token y %type */
%union
{
  char* ident;              /* Para los identificadores                      */
  int   ent;                /* Para constantes enteras                       */
  REFE  ref;                /* Para los no terminales con referencias dobles */
  EXPR  expre;              /* Para los no terminales que son expresiones    */
  int   aux;                /* Para los no terminales con un atributo simple */
}
/*********************************** Definiciones de los tokens del lenguaje */
%token 
/******************************************************* Palabras reservadas */

ELSE_ IF_ INT_ PRINT_ READ_ RETURN_ WHILE_ 

/*************************************************************** Separadores */

ACOR_ ALLA_ APAR_ CCOR_ CLLA_ CPAR_ COMA_ PUNTOCOMA_     

/**************** Definiciones de los simbolos de la gramatica con atributos */

/****************************************************** Tokens con atributos */
%token<ent>  CTE_ 
%token<ent> OPREL_ OPMUL_ OPADD_ OPASIG_ OPINCDEC_
%token<ident> ID_

/*********************************************** No terminales con atributos */
%type<aux>   lista_declaraciones  declaracion  decla_funcion  cab_funcion 
%type<aux>   param_formales  lista_param_form 
%type<aux>   param_actuales  lista_param_act 
%type<expre> expresion  exp_simple  termino  factor
%%
/*****************************************************************************/
/*    Esquemas de Traduccion Dirigidos por la Sintaxis que definen tanto la  */
/*  sintaxis como la semantica  y la generacion de codigo intermedio para    */
/*  el lenguaje MenosC.09.                                                   */
/*****************************************************************************/
programa
       : { dvar = 0; si = 0; carga_contexto(GLOBAL);

           /******************** Reserva de espacio para variables globales */
           arg1 = arg2 = cr_arg_nulo();
           res  = cr_arg_entero(-1);
           $<ref>$.ref1 = crea_lans(si);  
           emite(INCTOP, arg1, arg2, res);
           /************************ Salto al comienzo de la funcion "main" */
           arg1 = arg2 = cr_arg_nulo();
           res  = cr_arg_etiqueta(-1);
           $<ref>$.ref2 = crea_lans(si); 
           emite(CALL, arg1, arg2, res);
           res = cr_arg_nulo();
           emite(FIN, arg1, arg2,res);
         }

         lista_declaraciones
        
         { 
           if ($2 == 0) yyerror ("Error: el programa no tiene `main'");  
           /***** Completa la reserva de espacio para las variables globales */
           res = cr_arg_entero(dvar);
           completa_lans($<ref>1.ref1, res);
           /************* Completa el salto al comienzo de la funcion "main" */
           res = cr_arg_etiqueta($2);
           completa_lans($<ref>1.ref2, res);

           if (verbosidad) mostrar_tds();
           descarga_contexto(GLOBAL);
         }
       ;
/*****************************************************************************/
lista_declaraciones
       : declaracion

         { $$ = $1; }

       | lista_declaraciones declaracion

         { if ($1 == 0)      $$ = $2;
	   else if ($2 == 0) $$ = $1;
                else yyerror ("Error: el programa tiene mas de un `main'");
	 }
       ;
/*****************************************************************************/
declaracion
       : decla_variable

         { $$ = 0; }

       | decla_funcion

         { $$ = $1; }
       ;
/*****************************************************************************/
decla_variable
       : INT_ ID_ PUNTOCOMA_ 

         {
           if (!inserta_simbolo($2,VARIABLE,T_ENTERO,dvar,contexto,-1))
             yyerror ("Identificador repetido");  
           dvar += TALLA_ENTERO;
         }

       | INT_ ID_ ACOR_ CTE_ CCOR_ PUNTOCOMA_

         {
           if ($4 <= 0) yyerror("Error en la talla del array");
           if (!inserta_simbolo($2,VARIABLE,T_ARRAY,dvar,contexto, 
			      inserta_info_array(T_ENTERO, $4)))
             yyerror ("Identificador repetido");  
           dvar += $4 * TALLA_ENTERO;
         }
       ;
/*****************************************************************************/
decla_funcion
       : cab_funcion 

         {  $<aux>$ = dvar; dvar = 0;  }

         bloque

         { 
           $$ = $1;
           if (verbosidad) mostrar_tds(); 
           dvar = $<aux>2; descarga_contexto(LOCAL);  
         }
       ;
/*****************************************************************************/
cab_funcion
       : INT_ ID_  

         { 
           carga_contexto(LOCAL); dpar = TALLA_SEGENLACES;
         }

         APAR_ param_formales  CPAR_ 

         {
           if (!inserta_simbolo($2,FUNCION,T_ENTERO,si,GLOBAL,$5))
             yyerror ("Identificador repetido");
           if (!strcmp($2,"main")) $$ = si; else $$ = 0;
         }
       ;
/*****************************************************************************/
param_formales
       : /* cadena vacia */

         { $$ = inserta_info_dominio(-1, T_VACIO); }

       | lista_param_form

         { $$ = $1; }
       ;
/*****************************************************************************/
lista_param_form 
       : INT_ ID_ 

         {
           dpar += TALLA_ENTERO;
           if (!inserta_simbolo($2,PARAMETRO,T_ENTERO,-dpar,contexto,-1))
             yyerror ("Identificador repetido");    
           $$ = inserta_info_dominio(-1, T_ENTERO);
         }

       | INT_ ID_ COMA_ 

         {
           dpar += TALLA_ENTERO;
           if (!inserta_simbolo($2,PARAMETRO,T_ENTERO,-dpar,contexto,-1))
             yyerror ("Identificador repetido");    
         }

         lista_param_form 

         {
           $$ = inserta_info_dominio($5, T_ENTERO);
         }
       ;
/*****************************************************************************/
bloque 
       : { /*******************************  Carga de los enlaces de control */
           arg1 = arg2 = res = cr_arg_nulo();
           emite(PUSHFP, arg1, arg2, res);
           emite(FPTOP, arg1, arg2, res);
         } 

         ALLA_ decla_var_locales

         { /********* Reserva de espacio para Variables locales y temporales */
           arg1 = arg2 = cr_arg_nulo();
           res = cr_arg_entero(0);
           $<aux>$ = crea_lans(si);
           emite(INCTOP, arg1, arg2, res);
         }

         lista_instrucciones RETURN_ expresion CLLA_

         { INF x;

           /********* Completa la reserva de espacios para variables locales */
           res = cr_arg_entero(dvar);    
           completa_lans($<aux>4, res); 

	   x=obtener_info_funcion(-1); 
           if (($7.tipo != x.tipo))
             yyerror("Error de tipos en la expresion del `return'");

           /*************************************** guardar valor de retorno */
           res = cr_arg_posicion(contexto,-(x.tparam+3));
           arg2 = cr_arg_nulo();
           emite(EASIG, $7.pos, arg2, res);
  
           /*********** Libera el segmento de variables locales y temporales */
           arg1 = arg2 = res = cr_arg_nulo();
           emite(TOPFP, arg1, arg2, res);

           /***************************** Descarga de los enlaces de control */
           emite(FPPOP, arg1, arg2, res);
           /****************** Emite la instruccion RETURN */
           emite(RET, arg1, arg2, res);
	 }
       ;
/*****************************************************************************/
decla_var_locales
       : /* cadena vacia */
       | decla_var_locales decla_variable
       ;
/*****************************************************************************/
lista_instrucciones
       : /* instruccion vacia */
       | lista_instrucciones instruccion
       ;
/*****************************************************************************/
instruccion
       : ALLA_ lista_instrucciones CLLA_
       | inst_asignacion
       | inst_es
       | inst_if
       | inst_while
       ;
/*****************************************************************************/
inst_asignacion
       : ID_ OPASIG_ expresion PUNTOCOMA_

         { SIMB sim; int k;

           if ((k= busca_id($1)) < 0)
             yyerror("Identificador no declarado");
	   else sim = obtener_simbolo(k);
	   if (sim.tipo != $3.tipo)
             yyerror("Error de tipos en la asignacion");
           switch (sim.categoria) {
           case VARIABLE:
             res = cr_arg_posicion(sim.nivel, sim.desp);
             break;
           case PARAMETRO:
             res = cr_arg_posicion(sim.nivel, sim.desp);
             break;
           case FUNCION:
             yyerror("Error: asignacion a una funcion");
             break;
           }
           /*************************************** Asignaciones elementales */
           switch ($2) {
           case ASIG:
             arg2 = cr_arg_nulo();
             emite(EASIG, $3.pos, arg2, res);
             break;
           case MASASIG:
	     emite(ESUM, res, $3.pos, res);
             break;
           case MENOSASIG:
	     emite(EDIF, res, $3.pos, res);
             break;
           }
         }

       | ID_ ACOR_ expresion CCOR_ OPASIG_ expresion PUNTOCOMA_

         { SIMB sim; DIM d; int k;

           if ((k= busca_id($1)) < 0)
             yyerror("Identificador no declarado");
           else sim = obtener_simbolo(k);
           if ((sim.categoria != VARIABLE) || (sim.tipo != T_ARRAY))
             yyerror("El identificador debe ser una variable de tipo array");
           if ($3.tipo != T_ENTERO)
              yyerror("El indice del array debe ser un entero");
           d = obtener_array(sim.ref);
           if ($6.tipo != d.tipo)
             yyerror("Error de tipos en la asignacion");
           /*************************** Asignacion a un elemento de un array */
           arg1 = cr_arg_posicion(sim.nivel, sim.desp);
           switch ($5) {
           case ASIG:
             emite(EVA, arg1, $3.pos, $6.pos);
             break;
           case MASASIG:
	     res = cr_arg_posicion(contexto, crea_var_temp());
             emite(EAV, arg1, $3.pos, res);
	     emite(ESUM, res, $6.pos, res);
             emite(EVA, arg1, $3.pos, res);
             break;
           case MENOSASIG:
	     res = cr_arg_posicion(contexto, crea_var_temp());
             emite(EAV, arg1, $3.pos, res);
	     emite(EDIF, res, $6.pos, res);
             emite(EVA, arg1, $3.pos, res);
             break;
           }
         }
       ;
/*****************************************************************************/
inst_es
       : READ_ APAR_ ID_ CPAR_ PUNTOCOMA_

         { SIMB sim; int k;

           if ((k= busca_id($3)) < 0)
             yyerror("Identificador no declarado");
	   else sim = obtener_simbolo(k);
           if ((sim.categoria!=VARIABLE)&&(sim.tipo != T_ENTERO)) 
             yyerror("Debe ser una variable de tipo entero");
           /**************************************** Lectura de una variable */
           arg1 = arg2 = cr_arg_nulo();
           res = cr_arg_posicion(sim.nivel, sim.desp);
           emite(EREAD, arg1, arg2, res);
         }

       | PRINT_ APAR_ expresion CPAR_ PUNTOCOMA_

         {
           if ($3.tipo != T_ENTERO) 
             yyerror("El argumento del `write' debe ser entero");
           /************************************* Escritura de una expresion */
           arg1 = arg2 = cr_arg_nulo();
           emite(EWRITE, arg1, arg2, $3.pos);
         }  
       ;
/*****************************************************************************/
inst_if
       : IF_ APAR_ expresion CPAR_ 

         {
           if ($3.tipo != T_LOGICO)
             yyerror("La condicion debe ser de tipo logico");
           /********************************************** Salto condicional */
           arg2 = cr_arg_entero(0);
           res  = cr_arg_etiqueta(-1);
           $<aux>$ = crea_lans(si);
           emite(EIGUAL, $3.pos, arg2, res);
         }

         instruccion 

         { 
           res = cr_arg_etiqueta(si+1); 
           completa_lans($<aux>5, res);
           /******************************************** Salto incondicional */
           arg1 = arg2 = cr_arg_nulo();
           res  = cr_arg_etiqueta(-1);
           $<aux>$  = crea_lans(si);
           emite(GOTOS, arg1, arg2, res);
         }

         ELSE_ instruccion

         {
           res = cr_arg_etiqueta(si); 
           completa_lans($<aux>7, res);
	 }
       ;
/*****************************************************************************/
inst_while
       : { $<aux>$ = si; }

         WHILE_ APAR_  expresion CPAR_ 

         {
           if ($4.tipo != T_LOGICO)
             yyerror("La condicion debe ser de tipo logico");
           /********************************************** Salto condicional */
           arg2 = cr_arg_entero(0); res  = cr_arg_etiqueta(-1);
           $<aux>$ = crea_lans(si);
           emite(EIGUAL, $4.pos, arg2, res);
	 }

         instruccion 

         { /******************************************** Salto incondicional */
           arg1 = arg2 = cr_arg_nulo();
           res  = cr_arg_etiqueta($<aux>1);
           emite(GOTOS, arg1, arg2, res);
           /******************************** Completa el salto incondicional */
           res = cr_arg_etiqueta(si); 
           completa_lans($<aux>6, res);
         }
       ;
/*****************************************************************************/
expresion
       : exp_simple

         { $$ = $1; }

       | exp_simple OPREL_ exp_simple

         { int codop; $$.tipo = T_ERROR;

           if (($1.tipo != T_ENTERO) || ($3.tipo != T_ENTERO))
             yyerror("Los operandos deben ser enteros");
           $$.tipo = T_LOGICO;
	   $$.pos = cr_arg_posicion(contexto, crea_var_temp());
           switch($2){
             case DISTINTO:    codop = EDIST;  break;
             case IGUAL:       codop = EIGUAL; break;
             case MAYOR:       codop = EMAY;   break;
             case MENOR:       codop = EMEN;   break;
             case MAYORIG:     codop = EMAYEQ; break;
             case MENORIG:     codop = EMENEQ; break;
             default:                          break;
           }
           /*********************************************** Expresion = TRUE */
           arg1 = cr_arg_entero(1);
           arg2 = cr_arg_nulo();
           emite(EASIG, arg1, arg2, $$.pos);
           /************************************** Salto condicional si TRUE */
           res  = cr_arg_etiqueta(si+2);
           emite(codop, $1.pos, $3.pos, res);
           /********************************************** Expresion = FALSE */
           arg1 = cr_arg_entero(0);
           emite(EASIG, arg1, arg2, $$.pos);
         }
       ;
/*****************************************************************************/
exp_simple
       : termino

         { $$ = $1; }

       | exp_simple OPADD_ termino

         { int codop; $$.tipo = T_ERROR;

           if (($1.tipo != T_ENTERO) || ($3.tipo != T_ENTERO))
             yyerror("Error de tipo en la `exprisin simple'");
           $$.tipo = T_ENTERO;
           $$.pos = cr_arg_posicion(contexto, crea_var_temp());
           /***************** Expresion a partir de un operador aritmetico */
           emite($2, $1.pos, $3.pos, $$.pos);
         }
       ;
/*****************************************************************************/
termino
       : factor

         { $$ = $1; }

       | termino OPMUL_ factor

         { int codop; $$.tipo = T_ERROR;

           if (($1.tipo != T_ENTERO) || ($3.tipo != T_ENTERO))
             yyerror("Error de tipo en `termino'");
           $$.tipo = T_ENTERO;
           $$.pos = cr_arg_posicion(contexto, crea_var_temp());
            /***************** Expresion a partir de un operador aritmetico */
           emite($2, $1.pos, $3.pos, $$.pos);
         }
       ;
/*****************************************************************************/
factor
       : ID_

         { SIMB sim; int k;

           if ((k= busca_id($1)) < 0) {
             $$.tipo = T_ERROR;
             yyerror("Identificador no declarado");
	   }
	   else {sim = obtener_simbolo(k);
                 $$.tipo = sim.tipo;
	   }
           switch (sim.categoria) {
           case VARIABLE: 
             $$.pos = cr_arg_posicion(sim.nivel, sim.desp);
             break;
           case PARAMETRO:
             $$.pos = cr_arg_posicion(sim.nivel, sim.desp);
             break;
           case FUNCION:
             yyerror("Error en la sintaxis de la llamada a la funcion");
             break;
           }
         }

       | CTE_

         {
           $$.tipo = T_ENTERO;
           $$.pos = cr_arg_entero($1);    
         }

       | OPADD_ factor

         {
           $$.tipo = $2.tipo;
           if ($1 == MENOS) {
             if (($1 == MENOS)&&($$.tipo == T_ENTERO)) {
               /*************************** Cambio de signo en caso de MENOS */
               arg2 = cr_arg_nulo();   
               $$.pos = cr_arg_posicion(contexto, crea_var_temp());
               emite(ESIG, $2.pos, arg2, $$.pos);
               }
             else yyerror("Error de tipo en `factor'");
             }
           else $$.pos = $2.pos;
         }

       | OPINCDEC_ factor
         {
           $$.tipo = $2.tipo;
           if ($$.tipo != T_ENTERO)
             yyerror("Error de tipo en `factor'");
	   /******************************* Cambio de signo en caso de MENOS */
           arg2 = cr_arg_entero(1);
           $$.pos = cr_arg_posicion(contexto, crea_var_temp());
           emite($1, $2.pos, arg2, $2.pos);
           arg2 = cr_arg_nulo();
           emite(EASIG, $2.pos, arg2, $$.pos);
         }

       | APAR_ expresion CPAR_

         { $$ = $2;}

       | ID_ ACOR_ expresion CCOR_ 

         { 
           SIMB sim; DIM d; int k;

           if ((k= busca_id($1)) < 0)
             yyerror("Identificador no declarado");
           else sim = obtener_simbolo(k);
           if ((sim.categoria != VARIABLE) || (sim.tipo != T_ARRAY))
             yyerror("El identificador debe ser una variable de tipo array");
           if ($3.tipo != T_ENTERO)
              yyerror("El indice del array debe ser un entero");
           d = obtener_array(sim.ref);
           /**************************** Expresion como elemento de un array */
           arg1 = cr_arg_posicion(sim.nivel, sim.desp);
           $$.tipo = d.tipo;
           $$.pos  = cr_arg_posicion(contexto, crea_var_temp());
           emite(EAV, arg1, $3.pos, $$.pos);
         }

       | ID_ APAR_

         { /*********************** Reserva espacio para el valor de retorno */
           arg1 = arg2 = cr_arg_nulo();
           res  = cr_arg_entero(0);
           emite(EPUSH,arg2,arg2,res);
	 }

         param_actuales CPAR_

         { SIMB sim; int k; INF x;

           if ((k= busca_id($1)) < 0)
             yyerror("Identificador no declarado");
	   else sim = obtener_simbolo(k);
           if (sim.categoria != FUNCION) 
            yyerror("La variable no es una funcion");
	   x=obtener_info_funcion(k); 
           if (!compara_dominio(sim.ref, $4))
            yyerror("Error en el dominio de parametros actuales");
           /******************************************* Llamada a la funcion */
           arg1 = arg2 = cr_arg_nulo();
           res = cr_arg_etiqueta(sim.desp);
           emite(CALL, arg1, arg2, res);
           /***************************** Desapila el segmento de parametros */
           res = cr_arg_entero(x.tparam);
           emite(DECTOP, arg1, arg2, res);
           /************************** Desapila y asigna el valor de retorno */
           $$.tipo = sim.tipo;
	   $$.pos  = cr_arg_posicion(contexto, crea_var_temp());
           emite(EPOP, arg1, arg2, $$.pos);
         }
       ;
/*****************************************************************************/
param_actuales
        : /* cadena vacia */

          { $$ = inserta_info_dominio(-1, T_VACIO); }
 
        | lista_param_act

          { $$ = $1; }
        ;
/*****************************************************************************/
lista_param_act
       : expresion

         {
           if (($1.tipo != T_ENTERO) && ($1.tipo != T_LOGICO)) 
             yyerror("El tipo de los parametros debe ser de tipo simple");
           $$ = inserta_info_dominio(-1, $1.tipo);
           /************************************** Apila el parametro actual */
           arg1 =  arg2 = cr_arg_nulo();
           emite(EPUSH, arg1, arg2, $1.pos);
         }

       | expresion COMA_ lista_param_act

         {
           if (($1.tipo != T_ENTERO) && ($1.tipo != T_LOGICO)) 
             yyerror("El tipo de los parametros debe ser de tipo simple");
           $$ = inserta_info_dominio($3, $1.tipo);
           /************************************** Apila el parametro actual */
           arg1 =  arg2 = cr_arg_nulo();
           emite(EPUSH, arg1, arg2, $1.pos);
         }
       ;
/*****************************************************************************/
%%
/******************************************************************************
******************************************************************************/
