#include <stdio.h>
#include "../virtual/definiciones.h"

extern int yyparse();
extern FILE *yyin;
extern int yylineno;
extern int yydebug;
char *g_string_salida_error;
int g_max_string_salida_error;
int *n_l_leidas;
struct tipo_ctd *g_vector_salida;
int g_num_errores;
int error_max_leidas;

// Esta funcion escribe en un vector los errores tal y como se mostraran en la maquina virtual. Si llega a llenar
// el vector de caracteres deja de escribir. Es un limite generoso.
void yyerror(char *msg){
    int escritos;
    g_num_errores++;
    escritos = snprintf(g_string_salida_error, g_max_string_salida_error, "Error en linea %d: %s\n", yylineno - 1, msg);
    g_max_string_salida_error -= escritos;
    g_string_salida_error += escritos;
}

// Esta funcion llama al parser creado con flex y bison
// La funcion devuelve:
// -1 -> No se puede abrir el fichero con fopen. No deberia fallar nunca ya que la maquina virtual lo comprueba antes.
//       Pero puede fallar si alguien borra el fichero mientras ejecuta el programa.
// 0 -> No hay errores en el fichero de entrada.
// n > 0 -> El fichero de entrada tiene n errores.
// Los parametros:
// IN char *fichero_entrada -> La ruta completa del ficher de entrada
// OUT struct tipo_ctd *vector_salida -> El vector de salida con las instrucciones decodificadas. Al ser una funcion de C
//                                       no podemos usar clases. Es una estructura muy simple. Se necesita reservar la memoria antes
//                                       con un tamanyo MAXCODE
// OUT int *inst_leidas -> Devuelve por referencia el numero de instrucciones que tiene el programa. Se necesita reservar la memoria antes.
// OUT char *string_salida_error -> Devuelve un vector con los mensajes de error a mostrar. No se corre peligro de salir del rango
//                                  del vector. Se necesita reservar la memoria antes.
// IN int max_string_salida_error -> Debemos pasar a la funcion el tamanyo del vector string_salida_error.
//
// Se comprueba cualquier error sintactico, lexico o semantico, entre ellos:
// Que el fichero tenga las lineas ordenadas
// Que no tenga etiquetas apuntando a direcciones inexistentes
// Que hayan mas instrucciones de las soportadas (MAXCODE)

int parsea(char *fichero_entrada, struct tipo_ctd *vector_salida, int *inst_leidas, char *string_salida_error, int max_string_salida_error){
    int i, j, escritos, comp_orden;
    g_num_errores = 0;
    error_max_leidas = 0;
    n_l_leidas = inst_leidas;
    *n_l_leidas = 0;
    g_string_salida_error = string_salida_error;
    g_vector_salida = vector_salida;
    g_max_string_salida_error = max_string_salida_error;
    if ((yyin = fopen (fichero_entrada, "r")) == NULL) return -1;
    yylineno = 0;
    yyparse();
    
    //Ahora debemos comprobar que no hay errores:
    //Lineas desordenadas
    //Saltos a lineas que no existen
    
    comp_orden = 1;
    for(i = 0; i < *n_l_leidas; i++){
      if(comp_orden && g_num_errores == 0 && g_vector_salida[i].lin != i){
	g_num_errores++;  //El orden solo se comprueba hasta que falle una vez para evitar avalancha de errores
	comp_orden = 0;   //Ademas si hay otros errores no se comprobara
	escritos = snprintf(g_string_salida_error, g_max_string_salida_error, "Error en linea %d: Linea desordenada (Solo se avisara una vez)\n", i);
	g_max_string_salida_error -= escritos;
	g_string_salida_error += escritos;
      }
      for(j = 0; j < 3; j++) if(g_vector_salida[i].arg[j].tipo == ARG_ETIQUETA && g_vector_salida[i].arg[j].i_pos >= *n_l_leidas){
	g_num_errores++;
	escritos = snprintf(g_string_salida_error, g_max_string_salida_error, "Error en linea %d: La etiqueta apunta a una instruccion inexistente\n", i);
	g_max_string_salida_error -= escritos;
        g_string_salida_error += escritos;
      }
    }
    return g_num_errores; //Es el numero de errores que va a devolver
}
