/*****************************************************************************/
/*****************************************************************************/
/**  Utilidades para el manejo de la TDS, la gestion de memoria y la        **/
/**  generacion de codigo                                                   **/
/**                                                                         **/
/**  La version inicial estaba pensada para un LEB, tipo PASCAL, con pila **/
/**  y display. La version del 2008 reutiliza la anterior, pero elimina   **/
/**  la necesidad del control del display sustituyendolo por un fp.     **/
/**  Los prototipos de las funciones activas se pueden encontrar en el      **/
/**  header.h.                                                            **/
/**                     Jose Miguel Benedi, 2009-2010 <jbenedi@dsic.upv.es> **/
/*****************************************************************************/
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "header.h"
/*****************************************************************************/
/*********************** Constantes generales ********************************/
/*****************************************************************************/
/************************************** Constantes para la gestion de la TDS */
#define MaxTdB  10     /* Numero maximo de entradas en la Tabla de Bloques   */
#define MaxTdS 100     /* Numero maximo de entradas en la Tabla de Simbolos  */
#define MaxTdA  50     /* Numero maximo de entradas en la Tabla de Arrays    */
#define MaxTdR  50     /* Numero maximo de entradas en la Tabla de Registros */
#define MaxTdD  50     /* Numero maximo de entradas en la Tabla de Dominios  */
/************************************* Constantes para la gestion de memoria */
#define MaxCodigo 500          /* Talla maxima del segmento de instrucciones */
/************************************* Constantes para el manejo de las LANS */
#define MaxLans    50                    /* Talla maxima del vector de  LANS */
/******************** Constantes para los argumentos de las instrucciones 3D */
#define ARG_ENTERO    0
#define ARG_POSICION  1
#define ARG_ETIQUETA  2
#define ARG_NULO      3
/*****************************************************************************/
/*********************** Estructuras generales *******************************/
/*****************************************************************************/
/*************************************************** Estructuras para la TDS */
typedef struct tdbloq                  /* Elementos de la Tabla de Bloques   */
{
  int   ivar;                          /* Inicio de las variables en la TDS  */
  int   fvar;                          /* Final de las variables en la TDS   */
  int   ipar;                          /* Inicio de los parametros en la TDS */
  int   fpar;                          /* Final de los parametros en la TDS  */
  int   refe;                          /* Referencia a la funcion en la TDS  */
}TDBLOQ;

typedef struct tdsimb              /* Elementos de la Tabla de Simbolos      */
{
  char *nombre;                    /* Nombre del objeto                      */
  int   categoria;                 /* Categoria del objeto                   */
  int   tipo;                      /* Tipo del objeto                        */
  int   desp;                      /* Desplazamiento relativo en el segmento */
  int   nivel;                     /* Nivel del contexto                     */
  int   ref;                       /* Campo de referencia de usos multiples  */
}TDSIMB;

typedef struct tdarray                    /* Elementos de la Tabla de Arrays */
{
  int tipo;                               /* Tipo de los elementos del array */
  int lsup;                               /* Numero de elementos             */
}TDARRAY;

typedef struct tdreg                   /* Elementos de la Tabla de Registros */
{
  char *nombre;                        /* Nombre del campo                   */
  int   tipo;                          /* Tipo del campo                     */
  int   desp;                          /* Desplazamiento relativo en memoria */
  struct tdreg *siguiente;             /* Referencia al siguiente campo      */
}TDREG;

typedef struct tdomi                    /* Elementos de la Tabla de Dominios */
{
  int    tipo;                          /* Tipo del argumento                */
  struct tdomi *siguiente;              /* Referencia al siguiente argumento */
}TDOMI;
/************************************ Estructuras para la gestion de memoria */
typedef struct tipo_ctd   /* Estructura para las instrucciones del codigo 3D */
{ 
  int cop;                                   /* Codigo de la operacion       */
  TIPO_ARG arg1, arg2, res;                  /* Argumentos de la instruccion */
}TIPO_CTD;
/************************************ Estructuras para el manejo de las LANS */
typedef struct elans             /* Elementos de las LANS                    */
{
  int   dii;                     /* Direccion de la instruccion incompleta   */
  struct elans *siguiente;       /* Referencia a la siguiente dir.incompleta */
}ELANS;

/*****************************************************************************/
/*********************** Variables generales *********************************/
/*****************************************************************************/

/*************************** Variables globales de uso en todo el compilador */
int contexto;                 /* Contexto: ambito local o global             */
int dvar;                     /* Desplazamiento en el Segmento de Variables  */
int dpar;                     /* Desplazamiento en el Segmento de Parametros */
int si;                       /* Desplazamiento en el Segmento de Codigo     */
/***************************************************** Variables para la TDS */
TDBLOQ tdb[MaxTdB];         /* Tabla de Bloques                              */
TDSIMB tds[MaxTdS];         /* Tabla de Simbolos                             */
TDARRAY tda[MaxTdA];        /* Tabla de Arrays                               */
struct tdreg *tdr[MaxTdR];  /* Tabla de Registros                            */
struct tpform
{
  int           narg;                  /* Numero de argumentos de la funcion */
  struct tdomi *siguiente;             /* Referencia al siguiente argumento  */
} tdd[MaxTdD];              /* Tabla de Dominios                             */

int nivel = -1;             /* Nivel de anidamiento                          */
int ptdb  = 0;              /* Referencia al primer elemento libre de la TDB */
int ptds  = 0;              /* Referencia al primer elemento libre de la TDS */
int ptda  = 0;              /* Referencia al primer elemento libre de la TDA */
int ptdr  = 0;              /* Referencia al primer elemento libre de la TDR */
int ptdd  = 0;              /* Referencia al primer elemento libre de la TDD */
/************************************** Variables para la gestion de memoria */
char *cod_text[] =     /* Codigos de operacion asociados a las instrucciones */
{
"ESUM      ",       "EDIF      ",       "EMULT     ",       "EDIVI     ",
"RESTO     ",       "ESIG      ",       "EASIG     ",       "GOTOS     ",
"EIGUAL    ",       "EDIST     ",       "EMEN      ",       "EMAY      ",
"EMAYEQ    ",       "EMENEQ    ",       "EAV       ",       "EVA       ",
"EREAD     ",       "EWRITE    ",       "FIN       ",       "RET       ",
"CALL      ",       "EPUSH     ",       "EPOP      ",       "PUSHFP    ",
"FPPOP     ",       "FPTOP     ",       "TOPFP     ",       "INCTOP    ",
"DECTOP    "};

TIPO_CTD codigo[MaxCodigo];                            /* Segmento de codigo */
/************************************** Variables para el manejo de las LANS */
struct tlans
{
  int           ocupado;                             /* Flag de LANS ocupada */
  struct elans *siguiente;                           /* Referencia a la LANS */
} lans[MaxLans];                                     /* Vector de LANS       */

int inicio_lans = TRUE;    /* Flag para la inicializacion del vector de LANS */

/*****************************************************************************/
/*********************** Funciones de la libreria ****************************/
/*****************************************************************************/

/************************************* Operaciones para la gestion de la TDB */
void carga_contexto (int cont)
/* Crea el contexto necesario asi como las inicializalizaciones de la
   TDS y la TDB para un nuevo bloque con contexto (GLOBAL o LOCAL)
   definido en "cont". Adems, actualiza la variable global "contexto"
   a su nuevo valor.                                                         */

{
  nivel = cont;
  switch (nivel) {
  case 0:  contexto = GLOBAL; break;
  case 1:  contexto = LOCAL;  break;
  default: yyerror("<carga_contexto>: Error en la gestion del contexto");
  }

  if (nivel == ptdb) {
    if (ptdb < MaxTdB) {
      tdb[ptdb].ivar = tdb[ptdb].fvar = -1; tdb[ptdb].refe = -1;
      tdb[ptdb].ipar = tdb[ptdb].fpar = -1;
      if (nivel > 0) 
        if (ptds < MaxTdS) {
          tds[ptds].nombre = '\0'; tds[ptds].categoria = FUNCION; 
          tds[ptds].desp   = -1;   tds[ptds].tipo      = T_VACIO; 
          tds[ptds].ref    = -1;   tds[ptds].nivel     = -1;
          tdb[ptdb].refe = ptds; ptds++;
        }
        else yyerror("<carga_contexto>: Tabla de Simbolos completa");
      ptdb++;
    }
    else yyerror("<carga_contexto>: Tabla de Bloques completa");
  }
  else yyerror("<carga_contexto>: Error en la gestion del contexto");
}
/*****************************************************************************/
void descarga_contexto (int cont)
/* Libera en la TDB y la TDS el contexto asociado con el bloque actual
   con contexto (GLOBAL o LOCAL) definido en "cont".  Adems,
   actualiza la variable global "contexto" a su nuevo valor.                 */

{
  nivel = cont;
  if (nivel > 0) 
    if (nivel == (ptdb-1)) {
      ptds = tdb[nivel].refe+1;
      ptdb--; 
    }
    else yyerror("<descarga_contexto>: Error en la gestion del contexto");

  if (nivel >0 ) nivel--;
  switch (nivel) {
  case 0:  contexto = GLOBAL; break;
  case 1:  contexto = LOCAL;  break;
  default: yyerror("<descarga_contexto>: Error en la gestion del contexto");
  }
}
/************************************* Operaciones para la gestion de la TDS */
int inserta_info_array (int telem, int nelem)
/* Inserta en la Tabla de Arrays la informacion de un array cuyos
   elementos son de tipo "telem" y el numero de elementos es
   "nelem". Devuelve su referencia.                                          */
{
  if (ptda < MaxTdA) {
    tda[ptda].tipo = telem; tda[ptda].lsup = nelem; 
    ptda++;
    return ptda-1; 
  }
  else yyerror("<inserta_info_array>: Tabla de Arrays completa");
}

int inserta_info_dominio (int refe, int tipo)
/* Para un dominio existente referenciado por "refe", inserta en la
   Tabla de Dominios la informacion del "tipo" del parametro.  Si "ref
   = -1" entonces crea una nueva entrada en la tabla de dominios para
   el tipo de este parametro y devuelve su referencia.  Si la funcion
   no tiene parametros debe crearse un dominio vacio con: "refe = -1"
   y "tipo = T_NULO".                                                        */
{ TDOMI *p, *q, *r;

  p = (TDOMI *)malloc(sizeof(TDOMI)); 
  p->tipo = tipo; p->siguiente = NULL; 
  if (refe == -1) {
    if (ptdd < MaxTdD) {
      if (tipo == T_VACIO) {
	tdd[ptdd].narg = 0; tdd[ptdd].siguiente = NULL; 
      }
      else {
        tdd[ptdd].narg = 1; tdd[ptdd].siguiente = p;
      }
      ptdd++; 
      return ptdd-1;
    }
    else yyerror("<inserta_info_dominio>: Tabla de Dominios completa");
  }
  else if ((refe >= 0)&&(refe < ptdd)) {
    q = tdd[refe].siguiente; r = NULL;
    while (q != NULL) {
      r = q; q = q->siguiente; 
    }
    if ((r != NULL) && (tdd[refe].narg > 0)) {
      tdd[refe].narg++; r->siguiente = p;
    }
    else yyerror("<inserta_info_dominio>: El dominio esta definido vacio");
    return refe; 
  }
  else yyerror("<inserta_info_dominio>: Error acceso a la Tabla de Dominios");
}

int busca_id_nivel (char *nom, int n)            /* Es una funcion invisible */
/*  Busca un identificador de nombre "nom" en la TDS para el nivel "n".      */
/*  Devuelve su posicin en la TDS, o un "-1" en caso contrario.             */
{ int i, k = -1, b = TRUE;

  if ((n >= 0) && (n < ptdb)) {
    if (tdb[n].ivar >=0)
      for (i = tdb[n].ivar; ((i <= tdb[n].fvar) && b); i++) {
        if (strcmp(tds[i].nombre, nom) == 0) {
	  k = i; b = FALSE;
        }
      }
    if ((b) && (tdb[n].ipar >= 0))
      for (i = tdb[n].ipar; ((i <= tdb[n].fpar) && b); i++) {
       if (strcmp(tds[i].nombre, nom) == 0) {
	  k = i; b = FALSE;
       }
      }
  }
  else yyerror("<busca_id>: Nivel de anidamiento inadmisible");
  return k;
}

int busca_id (char *nom)
/* Busca un identificador de nombre "nom" en la TDS, accesible desde
   el contexto actual. Si lo encuentra, devuelve su indice (posicion)
   en la TDS; si no, devuelve un "-1".                                       */
{ int i, j, b;

 i = contexto; b= TRUE;  
  while ((i >= 0) && b) {
    if ((j =  busca_id_nivel(nom,i)) >= 0)  b = FALSE;
    else i--;
  }
  if (b) return -1;
  else return j;
}

int inserta_simbolo(char *nom,int clase,int tipo,int desp,int cont,int ref)
/* Inserta en la TDS toda la informacion asociada con un simbolo de:
   nombre "nom", clase "clase", tipo "tipo", desplazamiento relativo
   en el segmento correspondiente "desp", nivel de contexto "cont" y
   referencia a posibles subtablas "ref" (-1 si no referencia a otras
   subtablas).  Si el identificador ya existe en el contexto actual,
   devuelve el valor "FALSE" ("TRUE" en caso contrario).                     */
{ int k, i, ok = FALSE;

  nivel = cont;
  if ((k = busca_id_nivel(nom, nivel)) < 0) {
    if (ptds < MaxTdS) {
      ok = TRUE;
      if (clase != FUNCION) { i = ptds; ptds++; }
      else {
        i = tdb[nivel+1].refe;
        if (tdb[nivel].ivar < 0) tdb[nivel].ivar = i;
        tdb[nivel].fvar = i; 
      };
      tds[i].nombre = nom; tds[i].categoria = clase; tds[i].desp = desp;       
      tds[i].tipo  = tipo; tds[i].ref   = ref; tds[i].nivel = nivel;
      if (clase == VARIABLE) {
	if (tdb[nivel].ivar < 0) tdb[nivel].ivar = i; 
        tdb[nivel].fvar = i;
      }
      else if (clase == PARAMETRO) {
	if (tdb[nivel].ipar < 0) tdb[nivel].ipar = i; 
        tdb[nivel].fpar = i;
      }
    }
    else yyerror("<inserta_simbolo>: Tabla de Simbolos completa");
  }
  return ok;
}

SIMB obtener_simbolo (int ind)
/* Obtiene toda la informacion asociada con un simbolo referenciado
   por el indice "ind" en la TDS.                                            */
{ SIMB i;

  if ((ind >= 0) && (ind < ptds)) {
    i.categoria = tds[ind].categoria; i.tipo  = tds[ind].tipo; 
    i.desp      = tds[ind].desp;      i.nivel = tds[ind].nivel; 
    i.ref       = tds[ind].ref;  
  }
  else yyerror("<obtener_simbolo>: Acceso inadecuado a la Tabla de Simbolos");
  return i;
}

INF obtener_info_funcion (int ind)
/* Devuelve la informacion del tipo y el numero (talla) de parametros
   de la funcion referenciada por el indice "ind" en la TDS. Si
   "ind<0" entonces devuelve la informacion de la funcion asociada al
   contexto actual.                                                          */
{ INF i;
 if (ind < 0){
  i.tipo   =     tds[tdb[contexto].refe].tipo;
  i.tparam = tdd[tds[tdb[contexto].refe].ref].narg;
 }
 else {
  i.tipo   =     tds[ind].tipo;
  i.tparam = tdd[tds[ind].ref].narg;
 }
 return i;  
}

int es_main ()
/* Si la funcion actual es "main" devuelve "TRUE" ("FALSE en caso
   contrario).                                                               */
{
  nivel = contexto;
  if (strcmp(tds[tdb[nivel].refe].nombre, "main\0") == 0) return TRUE;
  return FALSE;
}

DIM obtener_array (int ref)
/* Devuelve toda la informacion asociada con un array referenciado por
  "ref" en la Tabla de Arrays.                                               */
{ DIM i;

  if ((ref >= 0) && (ref < ptda)) {
    i.tipo  = tda[ref].tipo; 
    i.lsup  = tda[ref].lsup;
  }
  else yyerror("<obtener_array>: Acceso inadecuado a la Tabla de Arrays");
  return i;
}

int compara_dominio (int refx, int refy)
/* Si los dominios referenciados por "refx" y "refy" no coinciden
   muestra un mensaje de error y devuelve "FALSE" ("TRUE" si son
   iguales).                                                                 */
{ TDOMI *p, *q; int c, b = FALSE; 

  if (((refx >= 0) && (refx < ptdd)) && ((refy >= 0) && (refy < ptdd))) {
    if (tdd[refx].narg == tdd[refy].narg) {
      p = tdd[refx].siguiente; q = tdd[refy].siguiente; c = TRUE;
      while ((p != NULL) && (q != NULL) && (c)) {
	if (p->tipo == q->tipo) {
	  p = p->siguiente; q = q->siguiente;
	}
	else {
	  c = FALSE; 
          yyerror("<compara_dominio>: Tipos de parametros incompatibles");
	}
      }
      if ((p == NULL) && (q == NULL) && (c))  b = TRUE;
    }
    else yyerror("<compara_dominio>: Numero erroneo de parametros");
  }
  else yyerror("<compara_dominio>: Acceso inadecuado a la Tabla de Dominios");
  return b;
}

void escribe_array(int ind)                             /* Operacion invisble*/
{
  printf("        ----------------------------------------\n");
  printf("        Tipo=      ");
  switch (tda[ind].tipo) {
  case T_VACIO:  printf ("%-9s","T_VACIO  "); break;
  case T_ENTERO: printf ("%-9s","T_ENTERO "); break;
  case T_LOGICO: printf ("%-9s","T_LOGICO "); break;
  case T_ARRAY:  printf ("%-9s","T_ARRAY  "); break; 
/*case T_RECORD: printf ("%-9s","T_RECORD "); break; */
  case T_ERROR:  printf ("%-9s","T_ERROR  "); break;
  }
  printf ("   Num_elem= %4d\n", tda[ind].lsup);
  printf("        ----------------------------------------\n");
}

void escribe_lista_arg (int ind)                        /* Operacion invisble*/
{ TDOMI *p;

  printf("        ----------------------------------------\n");
  printf("        Numero de argumentos:  %d\n", tdd[ind].narg);
  printf("        ----------------------------------------\n");
  if (tdd[ind].narg > 0) {
    p = tdd[ind].siguiente;
    while (p != NULL) {
      switch (p->tipo) {
      case T_VACIO:  printf ("         %-9s","T_VACIO  \n"); break;
      case T_ENTERO: printf ("         %-9s","T_ENTERO \n"); break;
      case T_LOGICO: printf ("         %-9s","T_LOGICO \n"); break;
      case T_ARRAY:  printf ("         %-9s","T_ARRAY  \n"); break;
/*    case T_RECORD: printf ("         %-9s","T_RECORD \n"); break; */
      case T_ERROR:  printf ("         %-9s","T_ERROR  \n"); break;
      }
      p = p->siguiente;
    }
    printf("        ----------------------------------------\n");
  }
}

void escribe_lista_var(int ind)                         /* Operacion invisble*/
{ int i;

  for (i = tdb[ind].ivar; i <= tdb[ind].fvar; i++) {
    printf("    %-14s",tds[i].nombre);
    switch (tds[i].categoria) {
    case VARIABLE:  printf (" %-13s","VARIABLE     "); break;
    case FUNCION:   printf (" %-13s","FUNCION      "); break;
    case PARAMETRO: printf (" %-13s","PARAMETRO    "); break;
    }
    switch (tds[i].tipo) {
    case T_VACIO:  printf (" %-9s","T_VACIO  "); break;
    case T_ENTERO: printf (" %-9s","T_ENTERO "); break;
    case T_LOGICO: printf (" %-9s","T_LOGICO "); break;
    case T_ARRAY:  printf (" %-9s","T_ARRAY  "); break;
/*  case T_RECORD: printf (" %-9s","T_RECORD "); break; */
    case T_ERROR:  printf (" %-9s","T_ERROR  "); break;
    }
    printf (" %d\n", tds[i].desp);
    switch (tds[i].tipo) { 
    case T_ARRAY: escribe_array(tds[i].ref); break;
/*  case T_RECORD: escribe_lista_campos(tds[i].ref); break; */
    }
    if (tds[i].categoria == FUNCION) 
      escribe_lista_arg(tds[i].ref);
  }
}

void escribe_lista_par(int ind)                         /* Operacion invisble*/
{ int i;

  for (i = tdb[ind].ipar; i <= tdb[ind].fpar; i++) {
    printf("    %-14s",tds[i].nombre);
    switch (tds[i].categoria) {
    case VARIABLE:  printf (" %-13s","VARIABLE     "); break;
    case FUNCION:   printf (" %-13s","FUNCION      "); break;
    case PARAMETRO: printf (" %-13s","PARAMETRO    "); break;
    }
    switch (tds[i].tipo) {
    case T_VACIO:  printf (" %-9s","T_VACIO  "); break;
    case T_ENTERO: printf (" %-9s","T_ENTERO "); break;
    case T_LOGICO: printf (" %-9s","T_LOGICO "); break;
    case T_ARRAY:  printf (" %-9s","T_ARRAY  "); break; 
/*  case T_RECORD: printf (" %-9s","T_RECORD "); break; */
    case T_ERROR:  printf (" %-9s","T_ERROR  "); break;
    }
    printf (" %d\n", tds[i].desp);
  }
}

void mostrar_tds ()
/* Muestra en pantalla toda la informacion de la TDS asociada con el
   contexto actual (GLOBAL o LOCAL)                                          */
{ int n = nivel; /*int n = nivel+1; */ 

  if ((n >=0) && (n < ptdb))
    if (n == 0) {
      printf("\n\n================================================\n");
      printf("Variables globales y funciones\n");
      if ((tdb[n].ivar >= 0)) {
        printf("================================================\n");
        printf("    Nombre         Categoria     Tipo      Desp.\n");
        printf("    --------------------------------------------\n");
        if (tdb[n].ivar >= 0) escribe_lista_var(n);
      }
      printf("================================================\n");
    }
    else {
      printf("\n\n================================================\n");
/*    printf("Nivel= %d; Nombre= %s;",n,tds[tdb[n].refe].nombre);*/
      printf("Nombre de la funcion = %s;\n", tds[tdb[n].refe].nombre);
      printf(" NArg= %d;",tdd[tds[tdb[n].refe].ref].narg);
      switch (tds[tdb[n].refe].tipo) {
        case T_VACIO:  printf (" Tipo= %s\n","T_VACIO  "); break;
        case T_ENTERO: printf (" Tipo= %s\n","T_ENTERO "); break;
        case T_LOGICO: printf (" Tipo= %s\n","T_LOGICO "); break;
        case T_ARRAY:  printf (" Tipo= %s\n","T_ARRAY  "); break;
/*      case T_RECORD: printf (" Tipo= %s\n","T_RECORD "); break; */
        case T_ERROR:  printf (" Tipo= %s\n","T_ID     "); break;
      }
      if ((tdb[n].ipar > 0) || (tdb[n].ivar > 0)) {
        printf("================================================\n");
        printf("    Nombre         Categoria     Tipo      Desp.\n");
        printf("    --------------------------------------------\n");
        if (tdb[n].ipar > 0) escribe_lista_par(n);
        if (tdb[n].ivar > 0) escribe_lista_var(n); 
      }
      printf("================================================\n");
    }
  else yyerror("Nivel de anidamiento inadmisible");
}

/*********************** Operaciones para la generacion de codigo intermedio */

TIPO_ARG cr_arg_nulo ()
/* Crea un argumento de una instruccion tres direcciones de tipo nulo.       */
{ TIPO_ARG k;
  
  k.tipo = ARG_NULO;
  return k;
}

TIPO_ARG cr_arg_entero (int valor)
/* Crea un argumento de una instruccion tres direcciones de tipo
   entero con "valor".                                                       */
{ TIPO_ARG k;

  k.tipo = ARG_ENTERO; k.val.i = valor;
  return k;
}

TIPO_ARG cr_arg_etiqueta (int valor)
/*  Crea el argumento de una instruccion tres direcciones de tipo
    etiqueta con la informacion de la direccion en "valor".                  */
{ TIPO_ARG k;
  
  k.tipo = ARG_ETIQUETA; k.val.e = valor;
  return k;
}

TIPO_ARG cr_arg_posicion (int n, int valor)
/*  Crea el argumento de una instruccion tres direcciones de tipo
    posicion con el contexto "n" y el desplazamiento "valor".                */
{ TIPO_ARG k;
  
  k.tipo = ARG_POSICION; k.val.p.niv = n; k.val.p.pos = valor;
  return k;
}

void emite (int cop, TIPO_ARG arg1, TIPO_ARG arg2, TIPO_ARG res)
/* Crea una instruccion tres direcciones con el codigo de operacion
   "cod" y los argumentos "arg1", "arg2" y "res", y la pone en la
   siguiente posicion libre (indicada por "si") del Segmento de
   Codigo. A continuacion, incrementa "si".                                  */
{
  if (si < MaxCodigo) {
    codigo[si].cop  = cop;  codigo[si].arg1 = arg1;
    codigo[si].arg2 = arg2; codigo[si].res  = res;
    si++;
  }
  else yyerror("<emite>: Segmento de Codigo completo");
}

int crea_var_temp ()
/*  Crea una variable temporal, de talla "1", en el segmento de
    variables del bloque actual y devuelve su desplazamiento
    relativo. A continuacn, incrementa "dvar = dvar + 1".                   */
{ int k;

  k = dvar; dvar += 1;
  return k;
}

void vuelca_codigo_arg(FILE *fd, TIPO_ARG arg)          /* Funcin invisible */
{
  switch(arg.tipo){
    case ARG_ENTERO:
      fprintf(fd,"i: %d  ",arg.val.i); break;
    case ARG_POSICION:
      fprintf(fd,"p: (%d,%d)  ",arg.val.p.niv, arg.val.p.pos); break;
    case ARG_ETIQUETA:
      fprintf(fd,"e: %d  ",arg.val.e); break;
    case ARG_NULO: fprintf(fd,"\t"); break;
    /* Falta el ARG_REAL ? */
    case ARG_REAL: 
      fprintf(fd,"r: %f ", 0.0); /* De todas formas no est preparado para ejecutarse ? */ break;
  }
}

void vuelca_codigo_ascii()
/* Vuelca el codigo generado, en modo texto, a un fichero cuyo nombre
   es el del fichero de entrada con la extension ".txt".                     */
{ FILE *fd; int ni; char *t;

  ni = strcspn(nom_fich,"."); t = (char *)malloc(ni+1); 
  strncpy(t, nom_fich, ni); t[ni] ='\0'; strcat(t,".c3d"); 
  fd = fopen (t, "w");
  for (ni = 0; ni < si; ni++) {
    fprintf(fd,"%3d %s\t", ni, cod_text[codigo[ni].cop]);
    vuelca_codigo_arg(fd, codigo[ni].arg1);
    fprintf(fd, ",  ");
    vuelca_codigo_arg(fd, codigo[ni].arg2);
    fprintf(fd, ",  ");
    vuelca_codigo_arg(fd, codigo[ni].res);
    fprintf(fd,"\n");
  }
  fclose (fd);
}

void vuelca_codigo_binario()
/* Vuelca el codigo generado, en modo binario, a un fichero cuyo
   nombre es el del fichero de entrada con la extension ".obj".              */
{ FILE *fd; int ni; char *t;

  ni = strcspn(nom_fich,"."); t = (char *)malloc(ni+1); 
  strncpy(t, nom_fich, ni); t[ni] = '\0'; strcat(t,".obj"); 
  fd = fopen (t, "w");
  for (ni = 0; ni < si; ni++) {
    fwrite(&codigo[ni], sizeof(TIPO_CTD), 1, fd);
  }
  fclose (fd);
}

/****************************** Operaciones para la manipulacion de las LANS */

int crea_lans (int d)
/* Crea una lista de argumentos no satisfechos para una instruccion
   incompleta cuya direccin es "d" y devuelve su referencia.                */
{ ELANS *p;  int i, b, j;

  if (inicio_lans == 1) {   /************ Inicializacion del vector de LANS  */
    for (j=0; j<MaxLans; ++j) {
      lans[j].ocupado = FALSE; lans[j].siguiente = NULL;
    }
    inicio_lans=FALSE;
  }

  p = (ELANS *)malloc(sizeof(ELANS)); 
  p->dii = d; p->siguiente = NULL; 

  i = 0; b = TRUE;
  while ((i < MaxLans) && b ) {
    if (lans[i].ocupado) i++;
    else {
      lans[i].ocupado = TRUE; lans[i].siguiente = p; b = FALSE;
    }    
  }
  if (b) yyerror("<crea_lans>: Vector de LANS completo");
  return i;
}

int fusiona_lans (int x, int y)
/* Fusiona dos listas de argumentos no satisfechos cuyas referencias
   son "x" e "y" y devuelve la referencia de la lista fusionada.             */
{ ELANS *p, *q;

 if (x < 0) {
   if (y < 0) return -1;
   else       return  y;
 }
 else
   if (y < 0) return  x;
   else {
     if ((x >= 0) && (x < MaxLans) && (lans[x].ocupado) &&
         (y >= 0) && (y < MaxLans) && (lans[y].ocupado)) {
       p = lans[x].siguiente; q = NULL;
       while (p != NULL) {
         q = p; p = p->siguiente;
         }
       if (q != NULL) q->siguiente = lans[y].siguiente; 
       else lans[x].siguiente = lans[y].siguiente; 
       lans[y].ocupado = FALSE; lans[y].siguiente = NULL;
       }
     else yyerror("<fusiona_lans>: Acceso inadecuado al vector de LANS");
     return x;
     }
}

void completa_lans (int x, TIPO_ARG arg)
/* Completa con el argumento "arg" el campo "res" de todas las
   instrucciones incompletas de la lista "x".                                */
{ ELANS *p, *q;
 
  if (x >= 0)
    if ((x < MaxLans) && (lans[x].ocupado)) {
      p = lans[x].siguiente;
      while (p != NULL) {
        codigo[p->dii].res = arg;
        q = p; p = p->siguiente; free(q);
       }
      lans[x].ocupado = FALSE; lans[x].siguiente = NULL;
    }
    else yyerror("<completa_lans>: Acceso inadecuado al vector de LANS");
}
/*****************************************************************************/
/*****************************************************************************/
/*********************** NO UTILIZADAS EN ESTE CURSO *************************/
/*****************************************************************************/
/*****************************************************************************/

int es_parte_izquierda (int ind, int n)
/*  Si el objeto referenciado por "ind" es el de la funcion que determina el */
/*  contexto actual "n", devuelve "TRUE" ("FALSE" en caso contrario).        */
{
  if ((tds[ind].nivel == n-1) && (tdb[n].refe == ind)) return TRUE;
  return FALSE;
}

/*****************************************************************************/
TDREG obtener_campo (int ref, char *nom)
/*               ojo esta funcin es pre-MenosC                              */
/* Obtiene toda la informacin asociada con un campo, de nombre "nom",
   referenciado por el ndice "ref" de un registro en la Tabla de
   Registros. En caso de error devuelve "tipo = T_Error" y "desp = -1".      */
{ TDREG i; TDREG *p; int b=TRUE;;

  if ((ref >= 0) && (ref < ptdr)) {
    p = tdr[ref]; 
    while ((p != NULL) && b) {
      if (strcmp(p->nombre,nom) == 0) b = FALSE;
      else p = p->siguiente; 
    }
    if (b) {
      i.tipo  = T_ERROR;
      i.desp  = -1;
    }
    else {
      i.tipo  = p->tipo; 
      i.desp  = p->desp;
    }
  }
  else yyerror("Acceso inadecuado a la Tabla de Registros");
  return i;
}
/*****************************************************************************/
int inserta_info_campo (int refe, char *nom, int tipo, int desp)
/*               ojo esta funcin es pre-MenosC                              */
/* Inserta en la Tabla de Registros, referenciada por "refe", la
   informacin de un determinado campo: nombre de campo "nombre", tipo
   de campo "tipo" y desplazamiento del campo "desp". Comprueba adems
   que el nombre del campo no este repetido en el registro.  
   Si "ref = -1" entonces crea una nueva entrada para este campo y
   la devuelve.                                                              */
{ TDREG *p, *q, *r;

  p = (TDREG *)malloc(sizeof(TDREG)); 
  p->nombre = nom; p->tipo = tipo; p->desp = desp; p->siguiente = NULL; 
  if (refe == -1) {
    if (ptdr < MaxTdR) {
      tdr[ptdr] = p; ptdr++; return ptdr-1;
    }
    else yyerror("Tabla de Registros completa");
  }
  else if ((refe >= 0)&&(refe < ptdr)) {
    q = tdr[refe];  r = NULL;
    while (q != NULL) {
      r = q; q = q->siguiente; 
      if (strcmp(r->nombre, nom) == 0)
	yyerror("Nombre de campo repetido");
    }
    if (r != NULL) r->siguiente = p;
    else tdr[refe] = p;
    return refe; 
  }
  else yyerror("Acceso inadecuado a la Tabla de Registros");
}
/*****************************************************************************/
void escribe_lista_campos(int ind)

{ TDREG *p;

  printf("        ----------------------------------------\n");
  printf("        Nombre         Tipo      Desp.\n");
  printf("        ----------------------------------------\n");
  p = tdr[ind];
  while (p != NULL) {
    printf("        %-14s",p->nombre);
    switch (p->tipo) {
    case T_VACIO:  printf (" %-9s","T_VACIO  "); break;
    case T_ENTERO: printf (" %-9s","T_ENTERO "); break;
    case T_LOGICO: printf (" %-9s","T_LOGICO "); break;
    case T_ARRAY:  printf (" %-9s","T_ARRAY  "); break; 
/*  case T_RECORD: printf (" %-9s","T_RECORD "); break; */
    case T_ERROR:  printf (" %-9s","T_ERROR  "); break;
    }
    printf (" %d\n", p->desp);
    p = p->siguiente;
  }
  printf("        ----------------------------------------\n");
}
/*****************************************************************************/
/*****************************************************************************/
