#include <QtGui>
#include "principal.h"

// Cargamos con un extern "C" el parser creado con flex-bison y compilado con gcc

extern "C" int parsea (char *fichero_entrada, struct tipo_ctd *vector_salida, int *inst_leidas, char *string_salida_error, int max_string_salida_error);


// Carga permite cargar un programa desde un archivo. En el caso que la carga sea existosa se borrara la memoria, se
// destruiran los breakpoints, se representaran las instrucciones de forma grafica y se modificaran las variables
// necesarias.
//
// Los parametros que admite son:
//
// IN QString archivoparaabrir -> La ruta al archivo que queremos abrir.
//
// El metodo pueden leer o escribir en las siguientes variables de clase:
//
//      OUT Principal::p_top: Se inicializa.
//      OUT Principal::totalbp: Se inicializa.
//      OUT Principal::inst_actual: Se inicializa.  
//      OUT Principal::p_fp: Se inicializa.
//      OUT Principal::ult_escr: Se pone a -1 (valor nulo).
//      OUT Principal::rep_mem: Si p_muestra y pila_estricta son true, modifica la representacion grafica de la memoria
//                              al modificar el puntero.
//      OUT Principal::list_mem: Se vacia la memoria
//      OUT Principal::rep_mem: Se limpia la representacion de la memoria
//      OUT Principal::list_inst: Se cargan las instrucciones en la lista
//      OUT Principal::rep_mem: Se representan las instrucciones cargadas
//      OUT Principal::ficheroactual: Se le asigna el valor del fichero abierto, para futuras recargas, ediciones o recompilaciones.

void Principal::cargar(QString archivoparaabrir){
    tipo_ctd v_entrada[MAXCODE];
    char errores_carga[MAXERR];
    int leidas, errores, i, j;
    QTreeWidgetItem *ninst;
    
    //Compruebo que el archivo existe y se puede abrir en modo lectura
    QFileInfo archivo(archivoparaabrir);
    if (!archivo.isReadable()) {
	QMessageBox::warning(this, tr("FalsPas"), tr("No puede leerse %1.").arg(archivo.fileName()));
	statusBar()->showMessage(tr("Error al acceder al archivo"), 5000);
	return;
    }
    
    QApplication::setOverrideCursor(Qt::WaitCursor);
    
    //Llamo al parser
    errores = parsea(archivo.canonicalFilePath().toUtf8().data(), v_entrada, &leidas, errores_carga, MAXERR);
    
    //Compruebo que la salida no contenga errores al abrir el fichero
    if(errores == -1){
	QApplication::restoreOverrideCursor();
	QMessageBox::warning(this, tr("FalsPas"), tr("No puede leerse %1.").arg(archivoparaabrir));
	statusBar()->showMessage(tr("Error al acceder al archivo"), 5000);
	return;
    }
    
    //Compruebo que la salida del parser no contenga errores logicos
    if(errores > 0){
        QApplication::restoreOverrideCursor();
        if(errores == 1){
	  QMessageBox::warning(this, tr("FalsPas"), tr("Se ha detectado un error en el fichero de entrada\nLea el panel de errores para mas informacion"));
	  textoerrores->setText(QString(errores_carga));
	  statusBar()->showMessage(tr("Error en el fichero de entrada"), 5000);
	}else{
	  QMessageBox::warning(this, tr("FalsPas"), tr("Se han detectado %1 errores en el fichero de entrada\nLea el panel de errores para mas informacion").arg(errores));
	  textoerrores->setText(QString(errores_carga));
	  statusBar()->showMessage(tr("Errores en el fichero de entrada"), 5000);
	} 
	return;
    }
    
    //A partir de aqui se supone que el fichero no contiene errores (se comprueban en parseo)
    total_inst = leidas;
    for(i = 0; i < MAXCODE; i++) list_inst.at(i)->setnull();
    for(i = 0; i < MAXMEM; i++){
      list_mem.at(i)->reset(); /* Borramos la memoria, su representacion y las instrucciones */
      rep_mem.at(i)->setText(3, QString());
      rep_mem.at(i)->setText(4, QString());
      rep_mem.at(i)->setText(5, QString());
      rep_mem.at(i)->setIcon(1, QIcon());
      rep_mem.at(i)->setIcon(2, QIcon());
    }
    
     //Ponemos la representacion de top y fp donde toca. Al final modificamos las variables de estado
    rep_mem.at(0)->setIcon(1, QIcon(":/iconos/flecha_azul.png"));
    rep_mem.at(0)->setIcon(2, QIcon(":/iconos/flecha_verde.png"));

    //Borramos los posibles items que tenga rep_inst
    while (!rep_inst.isEmpty()) delete rep_inst.takeFirst(); //Sinceramente no se si ese delete es necesario
    
    for(i = 0; i < leidas; i++){  //Cargamos las instrucciones y su representacion a la vez
      list_inst.at(i)->s_cop((t_cod_ins)v_entrada[i].cop);
      ninst = new QTreeWidgetItem();
      ninst->setText(2, QString("%1").arg(i));
      ninst->setText(3, textcodes.at(v_entrada[i].cop));
      for(j = 0; j < 3; j++){
	switch(v_entrada[i].arg[j].tipo){
	  case ARG_ENTERO:
	    list_inst.at(i)->s_int(j, v_entrada[i].arg[j].i_pos);
	    ninst->setText(4 + j, QString("I: %1").arg(v_entrada[i].arg[j].i_pos));
	    break;
	  case ARG_ETIQUETA:
	    list_inst.at(i)->s_eti(j, v_entrada[i].arg[j].i_pos);
	    ninst->setText(4 + j, QString("E: %1").arg(v_entrada[i].arg[j].i_pos));
	    break;
	  case ARG_REAL:
	    list_inst.at(i)->s_rea(j, v_entrada[i].arg[j].r);
	    ninst->setText(4 + j, QString("R: %1").arg(v_entrada[i].arg[j].r));
	    break;
	  case ARG_POSICION:
	    list_inst.at(i)->s_pos(j, v_entrada[i].arg[j].niv, v_entrada[i].arg[j].i_pos);
	    if(v_entrada[i].arg[j].niv) ninst->setText(4 + j, QString("P: 1,%1").arg(v_entrada[i].arg[j].i_pos));
	    else ninst->setText(4 + j, QString("P: 0,%1").arg(v_entrada[i].arg[j].i_pos));
	    ninst->setToolTip(4 + j, QString("Dir: %1").arg(v_entrada[i].arg[j].i_pos));
	    break;
	  case ARG_NULO:
	    ninst->setText(4 + j, QString(""));
	    break;
	} //Fin Switch
      } //Fin For j
      rep_inst.append(ninst);
    } //Fin For i
    
    //Dibujamos la flecha de posicion actual en la primera instruccion
    rep_inst.at(0)->setIcon(0, QIcon(style()->standardPixmap(QStyle::SP_ArrowRight)));
    //Asociamos la representacion de memoria al panel
    tree_inst->addTopLevelItems(rep_inst);
    
    //Ponemos un nuevo titulo a la ventana tras la carga
    setWindowTitle(tr("%1 - %2").arg(archivo.completeBaseName()).arg(tr("Falspas")));
    
    //Establecemos el nombre en una variable de clase para futuras recargas
    ficheroactual = archivoparaabrir;
    
    //Activamos las acciones necesarias
    accionrecargar->setEnabled(true);
    accionejecutar->setEnabled(true);
    accionsiguientepaso->setEnabled(true);
    accionparar->setEnabled(false);
    accioneditar->setEnabled(true);
    accioncompilar->setEnabled(true); 
    textosalida->setText(QString());
    textoerrores->setText(QString());
    
    //Modificamos las variables de la clase para inicializar el estado de la maquina
    totalbp = inst_actual = p_top = p_fp = 0;
    ult_escr = -1;
    statusBar()->showMessage(tr("Archivo %1 correctamente cargado").arg(QFileInfo(archivoparaabrir).completeBaseName()), 5000);
    QApplication::restoreOverrideCursor();
}
