#include "definiciones.h"
#include "principal.h"


// Este fichero contiene todos los metodos para manejar las instrucciones y las posiciones de memoria
// Los metodos cuyo nombre empieza por r_ leen un dato, mientras que los que empiezan por s_ lo escriben.
// El tipo de datos de un argumento o una instruccion lo define el metodo que lo ha escrito, es decir,
// usar instruccion->s_eti(0, 5); cambiara el tipo del primer argumento de la instruccion a ARG_ETIQUETA
// y le asignara el valor cinco.

c_tipo_ctd::c_tipo_ctd(){
    cop = NULO;
    breakpoint = false;
    v_arg[0].tipo_arg = v_arg[1].tipo_arg = v_arg[2].tipo_arg = ARG_NULO;
}

t_cod_ins c_tipo_ctd::r_cop(){
    return cop;
}

bool c_tipo_ctd::r_brp(){
    return breakpoint;
}

error_mem c_tipo_ctd::r_arg(int p_arg, t_cod_arg *p_cod){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    *p_cod = v_arg[p_arg].tipo_arg;
    return NO_ERROR;
}

error_mem c_tipo_ctd::r_int(int p_arg, int *p_val){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    if(v_arg[p_arg].tipo_arg != ARG_ENTERO) return ERROR_TIPO;
    *p_val = v_arg[p_arg].dato.i_e_pos_niv.i_e_pos;
    return NO_ERROR;
}


error_mem c_tipo_ctd::r_eti(int p_arg, int *p_val){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    if(v_arg[p_arg].tipo_arg != ARG_ETIQUETA) return ERROR_TIPO;
    *p_val = v_arg[p_arg].dato.i_e_pos_niv.i_e_pos;
    return NO_ERROR;
}

error_mem c_tipo_ctd::r_pos(int p_arg, bool *p_niv, int *p_pos){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    if(v_arg[p_arg].tipo_arg != ARG_POSICION) return ERROR_TIPO;
    *p_niv = v_arg[p_arg].dato.i_e_pos_niv.niv;
    *p_pos = v_arg[p_arg].dato.i_e_pos_niv.i_e_pos;
    return NO_ERROR;
}

error_mem c_tipo_ctd::r_rea(int p_arg, float *p_fval){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    if(v_arg[p_arg].tipo_arg != ARG_REAL) return ERROR_TIPO;
    *p_fval = v_arg[p_arg].dato.r;
    return NO_ERROR;
}

void c_tipo_ctd::s_cop(t_cod_ins p_cop){
    cop = p_cop;
}

void c_tipo_ctd::s_brp(bool p_brp){
    breakpoint = p_brp;
}

error_mem c_tipo_ctd::s_int(int p_arg, int p_val){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    v_arg[p_arg].tipo_arg = ARG_ENTERO;
    v_arg[p_arg].dato.i_e_pos_niv.i_e_pos = p_val;
    return NO_ERROR;
}

error_mem c_tipo_ctd::s_eti(int p_arg, int p_val){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    //if(p_val > MAXCODE) return ERROR_ARG; // Este error mejor que se encargue el parser
    v_arg[p_arg].tipo_arg = ARG_ETIQUETA;
    v_arg[p_arg].dato.i_e_pos_niv.i_e_pos = p_val;
    return NO_ERROR;
}

error_mem c_tipo_ctd::s_pos(int p_arg, bool p_niv, int p_pos){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    v_arg[p_arg].tipo_arg = ARG_POSICION;
    v_arg[p_arg].dato.i_e_pos_niv.niv = p_niv;
    v_arg[p_arg].dato.i_e_pos_niv.i_e_pos = p_pos;
    return NO_ERROR;
}

error_mem c_tipo_ctd::s_rea(int p_arg, float p_fval){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    v_arg[p_arg].tipo_arg = ARG_REAL;
    v_arg[p_arg].dato.r = p_fval;
    return NO_ERROR;
}

void c_tipo_ctd::setnull(){
    cop = NULO;
    v_arg[0].tipo_arg = v_arg[1].tipo_arg = v_arg[2].tipo_arg = ARG_NULO;
    breakpoint = false;
}

error_mem c_tipo_ctd::setnull(int p_arg){
    if(p_arg < 0 || p_arg > 2) return ERROR_ARG;
    v_arg[p_arg].tipo_arg = ARG_NULO;
    return NO_ERROR;
}    

c_tipo_mem::c_tipo_mem(){
    t = MEM_UNUSED;
}

tipo_mem c_tipo_mem::r_tip(){
    return t;
}

error_mem c_tipo_mem::r_int(int *p_val){
    if(t == MEM_UNUSED) return ERROR_VACIA;
    if(t != MEM_INT) return ERROR_TIPO;
    *p_val = celda.i;
    return NO_ERROR;
}

error_mem c_tipo_mem::r_rea(float *p_fval){
    if(t == MEM_UNUSED) return ERROR_VACIA;
    if(t != MEM_REAL) return ERROR_TIPO;
    *p_fval = celda.r;
    return NO_ERROR;
}

error_mem c_tipo_mem::r_fpo(int *p_val, int *p_refe){
    if(t == MEM_UNUSED) return ERROR_VACIA;
    if(t != MEM_FP) return ERROR_TIPO;
    *p_val = celda.fpo_des.p;
    *p_refe = celda.fpo_des.refe;
    return NO_ERROR;   
}

error_mem c_tipo_mem::r_des(int *p_val, int *p_refe){
    if(t == MEM_UNUSED) return ERROR_VACIA;
    if(t != MEM_DEST_S) return ERROR_TIPO;
    *p_val = celda.fpo_des.p;
    *p_refe = celda.fpo_des.refe;
    return NO_ERROR;   
}

void c_tipo_mem::s_int(int p_val){
    t = MEM_INT;
    celda.i = p_val;
}

void c_tipo_mem::s_rea(float p_fval){
    t = MEM_REAL;
    celda.r = p_fval;
}

void c_tipo_mem::s_fpo(int p_val, int p_refe){
    t = MEM_FP;
    celda.fpo_des.p = p_val;
    celda.fpo_des.refe = p_refe;
}

void c_tipo_mem::s_des(int p_val, int p_refe){
    t = MEM_DEST_S;
    celda.fpo_des.p = p_val;
    celda.fpo_des.refe = p_refe;
}

void c_tipo_mem::reset(){
    t = MEM_UNUSED;
}
    