#include <QtGui>
#include "dialogoopciones.h"


// El dialogo opciones permite seleccionar, tanto de modo grafico como escribiendo, la ruta donde se encuentre
// el editor y el compilador para poder usar las acciones de editar y compilar.
// Si el dialogo se cierra usando reject() no tendremos en cuenta ningun cambio.
// Si el dialogo se cierra usando accept() deberemos comprobar las siguientes variables publicas:
//    DialogoOpciones::QLineEdit *ed_editor: Donde deberiamos encontrar la ruta del editor
//    DialogoOpciones::QLineEdit *ed_compilador: Donde deberiamos encontrar la ruta del compilador
//    DialogoOpciones::int elem_leidos: Nos indicara cual de los dos elementos se han introducido de forma correcta:
//                                          0 -> Ninguno
//                                          1 -> Editor
//                                          2 -> Compilador
//                                          3 -> Tanto editor como compilador
// Los tres parametros son:
// IN Qwidget *padre: Puntero a la ventana padre del dialogo (this)
// IN QString act_editor: Ruta al actual editor
// IN QString act_comp: Ruta al actual compilador

DialogoOpciones::DialogoOpciones(QWidget *padre, QString act_editor, QString act_comp) : QDialog(padre){
    
    eti_editor = new QLabel(tr("Introducir ruta del &editor:"));
    ed_editor = new QLineEdit(act_editor);
    eti_compilador = new QLabel(tr("Introducir ruta del &compilador:"));
    ed_compilador = new QLineEdit(act_comp);
    eti_editor->setBuddy(ed_editor);
    eti_compilador->setBuddy(ed_compilador);
    acep = new QPushButton(QIcon(style()->standardPixmap(QStyle::SP_DialogOkButton)), tr("&Aceptar"));
    canc = new QPushButton(QIcon(style()->standardPixmap(QStyle::SP_DialogCancelButton)), tr("Ca&ncelar"));
    busca_edi = new QPushButton(QIcon(style()->standardPixmap(QStyle::SP_DirIcon)), "Buscar");
    busca_comp = new QPushButton(QIcon(style()->standardPixmap(QStyle::SP_DirIcon)), "Buscar");

    connect(acep, SIGNAL(clicked()), this, SLOT(comprueba()));
    connect(canc, SIGNAL(clicked()), this, SLOT(reject()));
    connect(busca_edi, SIGNAL(clicked()), this, SLOT(buscareditor()));
    connect(busca_comp, SIGNAL(clicked()), this, SLOT(buscarcompilador()));
    
    QHBoxLayout *compilador = new QHBoxLayout;
    QHBoxLayout *editor = new QHBoxLayout;
    QHBoxLayout *botones = new QHBoxLayout;
    compilador->addWidget(eti_compilador);
    compilador->addWidget(ed_compilador);
    compilador->addWidget(busca_comp);
    editor->addWidget(eti_editor);
    editor->addWidget(ed_editor);
    editor->addWidget(busca_edi);
    botones->addStretch();
    botones->addWidget(acep);
    botones->addWidget(canc);
    QVBoxLayout *ventana = new QVBoxLayout;
    ventana->addLayout(compilador);
    ventana->addLayout(editor);
    ventana->addLayout(botones);
    setLayout(ventana);
    setWindowTitle("Configuracion");
}


// Opciones auxiliares del dialogo

void DialogoOpciones::comprueba(){
  bool ed, comp;
  QFileInfo fil_editor(ed_editor->text());
  QFileInfo fil_compiler(ed_compilador->text());
  ed = !fil_editor.isExecutable();
  comp = !fil_compiler.isExecutable();
  
  if(ed && comp){
    QMessageBox::warning(this, tr("FalsPas"), tr("Los ficheros %1 y %2 no existen, no son ejecutables o no tiene los permisos necesarios").arg(ed_editor->text()).arg(ed_compilador->text()));
    elem_leidos = 0;
    accept();
  }
  if(comp){
    QMessageBox::warning(this, tr("FalsPas"), tr("El fichero %1 no existe, no es ejecutable o no tiene los permisos necesarios").arg(ed_compilador->text()));
    elem_leidos = 1;
    accept();
  }
  if(ed){
    QMessageBox::warning(this, tr("FalsPas"), tr("El fichero %1 no existe, no es ejecutable o no tiene los permisos necesarios").arg(ed_editor->text()));
    elem_leidos = 2;
    accept();
  }
  elem_leidos = 3;
  accept();
}

void DialogoOpciones::buscareditor(){
  ed_editor->setText(QFileDialog::getOpenFileName(this, tr("Elegir editor"), tr(""), tr("Todos los ficheros (*)")));
}

void DialogoOpciones::buscarcompilador(){
  ed_compilador->setText(QFileDialog::getOpenFileName(this, tr("Elegir compilador"), tr(""), tr("Todos los ficheros (*)")));
}