#include <climits>
#include <cfloat>
#include "principal.h"
#include "definiciones.h"
#include "introducirenteroej.h"
#include "introducirrealej.h"
#include "introducircaracterej.h"


// Este metodo decodifica cada instruccion y actua en consecuencia.
// Sus parametros son:
//   IN c_tipo_ctd *inst_exe: La instruccion que queremos ejecutar
//   IN bool muestra: Si la variable es true los cambios que provoque la instruccion se mostraran en la representacion grafica
//                    inmediatamente. En caso contrario no se representaran.
//   OUT int *salto: Variable de salida por referencia. Tiene el significado siguiente:
//                                 Si salto es -3 -> El programa ha llegado a la instruccion FIN
//                                 Si salto es -2 -> Se ha ejecutado reject() en algun dialogo de peticion al usuario
//                                 Si salto es -1 -> Deberemos sumar 1 al contador de programas (inst_actual)
//                                 Si salto es 0 o mayor -> Deberemos saltar a dicho numero de instruccion
//   OUT error_mem *err: Vector que hemos de pasar ya declarado donde guardaremos los posibles errores que ocurran en la ejecucion
//                       de la instruccion. Errores en las posiciones 0, 1 y 2 implican los respectivos argumentos. Errores en
//                       posiciones superiores pueden ser errores en calculos intermedios de la instruccion.
//
// El metodo lee las variables Principal::p_top y Principal::p_fp y usa los metodos Principal::recupera_op,
//  Principal::leer_mem, Principal::escribe_mem, Principal::set_top y Principal::set_fp.
//    

void Principal::ej_inst(c_tipo_ctd *inst_exe, bool muestra, error_mem *err, int *salto){
  int op1, op2, dato;
  float fop1, fop2, fdato;
  bool niv;
  op1 = op2 = dato = 0;
  fop1 = fop2 = fdato = 0.0;
  niv = false;
  *salto = -1;
  ult_escr = -1;
  for(int i = 0; i < MAXERRVEC; i++) err[i] = NO_ERROR;
  
  switch(inst_exe->r_cop()){
    case ESUM:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      err[2] = escribe_mem(2, inst_exe, op1 + op2, muestra); break;
    case RSUM:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      err[2] = escribe_mem(2, inst_exe, fop1 + fop2, muestra); break;
    case EDIF:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      err[2] = escribe_mem(2, inst_exe, op1 - op2, muestra); break;
    case RDIF:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      err[2] = escribe_mem(2, inst_exe, fop1 - fop2, muestra); break;
    case EMULT:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      err[2] = escribe_mem(2, inst_exe, op1 * op2, muestra); break;
    case RMULT:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      err[2] = escribe_mem(2, inst_exe, fop1 * fop2, muestra); break;
    case EDIVI:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2); /* Ojo con las divisiones entre 0 */
      if(op2) err[2] = escribe_mem(2, inst_exe, op1 / op2, muestra);
      else{
	err[5] = ERROR_DIV_CERO;
	err[2] = escribe_mem(2, inst_exe, 0, muestra);
      } break;
    case RDIVI:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2); /* Ojo con las divisiones entre 0 */
      if(op2) err[2] = escribe_mem(2, inst_exe, fop1 / fop2, muestra);
      else{
	err[5] = ERROR_DIV_CERO;
	err[2] = escribe_mem(2, inst_exe, (float) 0.0, muestra);
      } break;
    case RESTO:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2); /* Ojo con las divisiones entre 0 */
      if(op2) err[2] = escribe_mem(2, inst_exe, op1 % op2, muestra);
      else{
	err[3] = ERROR_DIV_CERO;
	err[2] = escribe_mem(2, inst_exe, 0, muestra);
      } break;
    case CONV:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[2] = escribe_mem(2, inst_exe, (float) op1, muestra); break;
    case TRUNC:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[2] = escribe_mem(2, inst_exe, (int) fop1, muestra); break;
    case ESIG:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[2] = escribe_mem(2, inst_exe, -op1, muestra); break;
    case RSIG:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[2] = escribe_mem(2, inst_exe, -fop1, muestra); break;
    case EASIG:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[2] = escribe_mem(2, inst_exe, op1, muestra); break;
    case RASIG:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[2] = escribe_mem(2, inst_exe, fop1, muestra); break;
    case GOTOS:
      err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED /*No lee en memoria*/); break;
    case EIGUAL:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      if(op1 == op2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case RIGUAL:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      if(fop1 == fop2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case EDIST:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      if(op1 != op2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case RDIST:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      if(fop1 != fop2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case EMEN:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      if(op1 < op2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case RMEN:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      if(fop1 < fop2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case EMAY:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      if(op1 > op2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case RMAY:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      if(fop1 > fop2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case EMAYEQ:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      if(op1 >= op2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case RMAYEQ:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      if(fop1 >= fop2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case EMENEQ:
      err[0] = recupera_op(0, inst_exe, &op1);
      err[1] = recupera_op(1, inst_exe, &op2);
      if(op1 <= op2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case RMENEQ:
      err[0] = recupera_op(0, inst_exe, &fop1);
      err[1] = recupera_op(1, inst_exe, &fop2);
      if(fop1 <= fop2) err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; // El posible error lo ha comprobado el parser
    case EAV: // p_arg3:=p_arg1[p_i_arg2]
      err[0] = recupera_op(0, inst_exe, &niv, &op1); /* El primer parametro es un puntero. Lo comprueba el parser */
      err[1] = recupera_op(1, inst_exe, &op2); // Esto es un entero o un puntero a entero, el desplazamiento
      err[3] = leer_mem(niv, op1 + op2, &dato); // Err4 solo se usa accediendo a vectores. Leemos el dato del vector
      err[2] = escribe_mem(2, inst_exe, dato, muestra); break; // Escribimos el dato en la posicion de argumento 
    case RAV: // p_arg3:=p_arg1[p_i_arg2]
      err[0] = recupera_op(0, inst_exe, &niv, &op1); /* El primer parametro es un puntero. Lo comprueba el parser */
      err[1] = recupera_op(1, inst_exe, &op2); // Esto es un entero o un puntero a entero, el desplazamiento
      err[3] = leer_mem(niv, op1 + op2, &fdato); // Err4 solo se usa accediendo a vectores. Leemos el dato del vector
      err[2] = escribe_mem(2, inst_exe, fdato, muestra); break; // Escribimos el dato en la posicion de argumento 3.
    case EVA: // p_arg1[p_i_arg2]:=p_arg3
      err[0] = recupera_op(0, inst_exe, &niv, &op1); /* El primer parametro es un puntero. Lo comprueba el parser */
      err[1] = recupera_op(1, inst_exe, &op2); // Entero o puntero a entero. El desplazamiento
      err[2] = recupera_op(2, inst_exe, &dato); // Es un puntero a entero. Debería poder ser también un entero, pero no se contempla
      err[3] = escribe_mem(niv, op1 + op2, dato, muestra); break;
    case RVA: // p_arg1[p_i_arg2]:=p_r_arg3
      err[0] = recupera_op(0, inst_exe, &niv, &op1); /* El primer parametro es un puntero. Lo comprueba el parser */
      err[1] = recupera_op(1, inst_exe, &op2); // Entero o puntero a entero. El desplazamiento
      err[2] = recupera_op(2, inst_exe, &fdato); // Es un puntero a real. Debería poder ser también un real, pero no se contempla
      err[3] = escribe_mem(niv, op1 + op2, fdato, muestra); break;
    case EREAD:
      {
	IntroducirEnteroEj ventanaent(this);
	if(ventanaent.exec()) op2 = ventanaent.edi->text().toInt();
	else{
	  op2 = 0;
	  *salto = -2;
	} //STOP in the name of love
	err[2] = recupera_op(2, inst_exe, &niv, &op1);
	err[3] = escribe_mem(niv, op1, op2, muestra);
      } break;
    case RREAD:
      {
	IntroducirRealEj ventanarea(this);
	if(ventanarea.exec()) fop2 = ventanarea.edi->text().toFloat();
	else{
	  fop2 = 0.0;
	  *salto = -2;
	} //STOP in the name of love
	err[2] = recupera_op(2, inst_exe, &niv, &op1);
	err[3] = escribe_mem(niv, op1, fop2, muestra);
      } break;
    case EWRITE:
      err[2] = recupera_op(2, inst_exe, &op1); // Hay que probar con algo que no sea append
      textosalida->setText(textosalida->toPlainText().append(QString("%1").arg(op1))); break;
    case RWRITE:
      err[2] = recupera_op(2, inst_exe, &fop1); // Hay que probar con algo que no sea append
      textosalida->setText(textosalida->toPlainText().append(QString("%1").arg(fop1))); break;
    case CREAD:
      {
	IntroducirCaracterEj ventanachar(this);
	if(ventanachar.exec()) op2 = (int)ventanachar.edi->text().at(0).toAscii();
	else{
	  op2 = 0;
	  *salto = -2;
	} //STOP in the name of love
	err[2] = recupera_op(2, inst_exe, &niv, &op1);
	err[3] = escribe_mem(niv, op1, op2, muestra);
      } break;
    case CWRITE:
      err[2] = recupera_op(2, inst_exe, &op1); // Hay que probar con algo que no sea append
      if(op1 == 10 || (op1 >= 32 && op1 <= 126)) textosalida->setText(textosalida->toPlainText().append(QString("%1").arg(QChar(op1))));
      else err[5] = ERROR_CWRITE; break;
    case FIN:
      *salto = -3; break; /* Es necesario? */
    case RET:
      err[3] = leer_mem(p_top - 1, salto, MEM_DEST_S); // Podria estar vacia la posicion
      err[4] = set_top(p_top - 1, muestra); break;
    case CALL:
      err[4] = set_top(p_top + 1, muestra);
      err[3] = escribe_mem(p_top - 1, inst_actual + 1, muestra, MEM_DEST_S, inst_actual); // No puede dar error al escribir dado que daria error antes un settop
      err[2] = recupera_op(2, inst_exe, salto, MEM_UNUSED); break; //El error lo comprueba el parser
    case EPUSH:
      err[2] = recupera_op(2, inst_exe, &op1);
      err[4] = set_top(p_top + 1, muestra);
      err[3] = escribe_mem(p_top - 1, op1, muestra); break; // No puede dar error, lo daria antes el settop
    case RPUSH:
      err[2] = recupera_op(2, inst_exe, &fop1);
      err[3] = set_top(p_top + 1, muestra); // No puede dar error, lo daria antes el settop
      err[0] = escribe_mem(p_top - 1, fop1, muestra); break;
    case EPOP:
      err[4] = leer_mem(p_top - 1, &op1);
      err[3] = set_top(p_top - 1, muestra);
      err[2] = escribe_mem(2, inst_exe, op1, muestra); break;
    case RPOP:
      err[4] = leer_mem(p_top - 1, &fop1);
      err[3] = set_top(p_top - 1, muestra);
      err[2] = escribe_mem(2, inst_exe, fop1, muestra); break;
    case PUSHFP:
      err[4] = set_top(p_top + 1, muestra);
      err[3] = escribe_mem(p_top - 1, p_fp, muestra, MEM_FP, inst_actual); break;
    case FPPOP:
      err[4] = leer_mem(p_top - 1, &op1, MEM_FP);
      err[6] = set_top(p_top - 1, muestra);
      err[5] = set_fp(op1, muestra); break;
    case FPTOP:
      err[3] = set_fp(p_top, muestra); break; // No puede dar error
    case TOPFP:
      err[3] = set_top(p_fp, muestra); break; // No puede dar error
    case INCTOP:
      err[0] = recupera_op(2, inst_exe, &op1); //El error lo comprueba el parser
      err[2] = set_top(p_top + op1, muestra); break;
    case DECTOP:
      err[0] = recupera_op(2, inst_exe, &op1); //El error lo comprueba el parser
      err[2] = set_top(p_top - op1, muestra); break;
    default:
      break;
  } //Cierra switch
} //Cierra metodo


// El metodo escribir_warnings escribe por el panel de errores los warnings existentes en el vector de errores.
// Esta funcion debera llamarse tras Principal::ej_inst pasandole como parametros:
//    IN error_mem *err: Un vector de errores escrito por el metodo Principal::ej_inst.
//    IN int inst_act: La instruccion que se ha ejecutado con anterioridad para poder indicar de forma mas precisa el error.

void Principal::escribir_warnings(error_mem *err, int inst_act){
  for(int i = 0; i < MAXERRVEC; i++){
    switch(err[i]){
      case ERROR_TIPO:
	if(i < 3) textoerrores->append(QString("Warning en la instruccion %1: Error de tipos en el parametro %2\nProbablemente su programa no funcione correctamente\n").arg(inst_act).arg(i));
	else textoerrores->append(QString("Warning en la instruccion %1: Error de tipos\nProbablemente su programa no funcione correctamente\n").arg(inst_act));
	break;
      case ERROR_NOEXISTE:
	if(i < 3) textoerrores->append(QString("Warning en la instruccion %1: Se lee o se escribe fuera del area de memoria en el parametro %2\nProbablemente su programa no funcione correctamente\n").arg(inst_act).arg(i));
	else textoerrores->append(QString("Warning en la instruccion %1: Se lee o se escribe fuera del area de memoria\nProbablemente su programa no funcione correctamente\n").arg(inst_act));
	break;
      case ERROR_VACIA:
	if(i < 3) textoerrores->append(QString("Warning en la instruccion %1: Se intenta leer de una posicion de memoria no inicializada en el parametro %2\nProbablemente su programa no funcione correctamente\n").arg(inst_act).arg(i));
	else textoerrores->append(QString("Warning en la instruccion %1: Se intenta leer de una posicion de memoria no inicializada\nProbablemente su programa no funcione correctamente\n").arg(inst_act));
	break;
      case ERROR_ARG:
	if(i < 3) textoerrores->append(QString("Warning en la instruccion %1: Se intenta leer de un parametro %2 no valido de la instruccion\nProbablemente su programa no funcione correctamente\n").arg(inst_act).arg(i));
	else textoerrores->append(QString("Warning en la instruccion %1: Se intenta leer de un parametro no valido de la instruccion\nProbablemente su programa no funcione correctamente\n").arg(inst_act));
	break;
      case ERROR_DIV_CERO:
	textoerrores->append(QString("Warning en la instruccion %1: Se divide entre cero\nProbablemente su programa no funcione correctamente\n").arg(inst_act));
	break;
      case ERROR_CWRITE:
	textoerrores->append(QString("Warning en la instruccion %1: Se intenta escribir un numero no representable\nProbablemente su programa no funcione correctamente\n").arg(inst_act));
	break;
      case ERROR_SUP_TOP:
	if(i < 3) textoerrores->append(QString("Warning en la instruccion %1: Se intenta leer o escribir por encima del puntero a pila en la instruccion %2\nProbablemente su programa no funcione correctamente\n").arg(inst_act).arg(i));
	else textoerrores->append(QString("Warning en la instruccion %1: Se intenta leer o escribir por encima del puntero a pila\nProbablemente su programa no funcione correctamente\n").arg(inst_act));
	break;	      
      default: //NO_ERROR
	break;
    } //Fin Switch
  } //Fin For
} //Fin Metodo


// El metodo set_dir_fisicas modifica la representacion de las direcciones fisicas de memoria que se muestran en el panel de instrucciones.
// Se llama principalmente desde el metodo Principal::set_fp, ya que cambiar el puntero a marco implica cambiar la direccion fisicas
// de todas las variables globales.
// Esta funcion lee la variable Principal::list_inst y modifica Principal::rep_inst

void Principal::set_dir_fisicas(){
  int cont1, cont2, pos;
  bool niv;
  for(cont1 = 0; cont1 <= total_inst; cont1++){ //Reestablece las direcciones fisicas
    for(cont2 = 0; cont2 < 3; cont2++){
      if(NO_ERROR == list_inst.at(cont1)->r_pos(cont2, &niv, &pos))
	if(niv) rep_inst.at(cont1)->setToolTip(4 + cont2, QString("Direccion: %1").arg(pos + p_fp)); //Else no es necesario
    } //Fin For cont1
  } //Fin For cont2
} //Fin metodo
