#include <QtGui>
#include "principal.h"

void Principal::ejecutar(){
    
    /* Deberíamos mirrar en tres etapas:
       -Comprobar el estado, quitar top, fp, inst_act y ult_escr de la representacion
       -Ejecutar hasta encontrar un breakpoint o una instruccion fin
       -Reestablecer el estado (top, fp, inst_act y ult_escr) en la representacion */
    
    /* Un breakpoint se comprueba DESPUES de ejecutarlo. Si no nunca se avanzaria */
    
    /* Tener estas acciones activas no es recomendable, ya que no funcionarian */
    accionejecutar->setEnabled(false);
    accionparar->setEnabled(false);
    accionsiguientepaso->setEnabled(false);
    accionabrir->setEnabled(false);
    accionrecargar->setEnabled(false);
    accioncompilar->setEnabled(false);
    accioneditar->setEnabled(false);
    
    /* Quitamos la representacion del p_fp y el p_top */
    rep_mem.at(p_fp)->setIcon(2, QIcon());
    rep_mem.at(p_top)->setIcon(1, QIcon());
    
    /* Quitamos la representación del posible último escrito */
    if(ult_escr != -1) for(int i = 0; i < 6; i++) rep_mem.at(ult_escr)->setBackground(i, QBrush(Qt::NoBrush));
    
    /* Quitamos la indicacion de inst_actual */
    rep_inst.at(inst_actual)->setIcon(0, QIcon());
    
    /*Ahora ejecutamos en un while del que solo saldremos
      -Cuando se apriete el boton parar en un read (salto = -2)
      -Cuando llegue una instruccion fin (salto = -3)
      -Cuando llegue un breakpoint, que se comprobara al final
      Si una instruccion no salta entonces salto = -1
      Al comprobar el while si salto = -5 entonces hay condicion
      de salir correctamente o con el PC desbocado */
    int salto;
    error_mem err[MAXERRVEC];
    
    do{
      
      //c_tipo_ctd *inst_exe = list_inst.at(inst_actual);
      ej_inst(list_inst.at(inst_actual), false /*No debe mostrarse aqui*/, err, &salto);
      
      escribir_warnings(err, inst_actual);
      
      switch(salto){
	case -3: //Se acabo el programa bien con una instruccion FIN
	  textoerrores->append(QString("El programa ha finalizado correctamente\nPara volverlo a ejecutar debe reiniciarlo"));
	  rep_inst.at(inst_actual)->setIcon(0, QIcon(style()->standardPixmap(QStyle::SP_ArrowRight)));
	  accionparar->setEnabled(true);
	  accionabrir->setEnabled(true);
	  accionrecargar->setEnabled(true);
	  accioncompilar->setEnabled(true);
	  accioneditar->setEnabled(true); break; //Debemos desactivar las acciones paso a paso, ejecutar y recuperar la memoria
	case -2:
	  break;
	case -1:
	  inst_actual++; break; //No era una instruccion de salto
	default:
	  inst_actual = salto; break; //Es un salto
      }
      
      if(inst_actual >= total_inst || inst_actual < 0){
	textoerrores->append(QString("Error. Ha modificado el P.C. a una posicion que no existe en su programa (%1)\nProbablemente su programa no funcione correctamente\nDebe reiniciarlo para continuar").arg(inst_actual));
	accionparar->setEnabled(true);
	accionabrir->setEnabled(true);
	accionrecargar->setEnabled(true);
	accioncompilar->setEnabled(true);
	accioneditar->setEnabled(true);
	salto = -5;
      }
      
    }while(salto != -5 && salto != -2  && salto != -3 && !list_inst.at(inst_actual)->r_brp()); //Se ejecuta con cortocircuito
    
    /* Ahora aquí hay que dejar todo como estaba. Volcar entera la memoria, p_top, p_fp, inst_act y ultima escritura */
    /* Y cambiar la indicacion de direccion fisica */
    
    set_top(p_top, true);
    set_fp(p_fp, true);
    
    set_dir_fisicas(); //Puede haber cambiado el FP y no estamos representando los cambios en el bucle
       
    for(int i = 0; i < MAXMEM; i++){ //Reestablece el panel de memoria
	tipo_mem mem_aux = list_mem.at(i)->r_tip();
	int i_aux, i_aux_ref;
	float f_aux;
	switch(mem_aux){
	  case MEM_INT:
	    list_mem.at(i)->r_int(&i_aux);
	    rep_mem.at(i)->setText(3, QString("E:"));
	    rep_mem.at(i)->setText(4, QString("%1").arg(i_aux));
	    if(i_aux >= 32 && i_aux <= 126) rep_mem.at(i)->setText(5, QString(QChar(i_aux)));
	    else rep_mem.at(i)->setText(5, QString());
	    break;
	  case MEM_REAL:
	    list_mem.at(i)->r_rea(&f_aux);
	    rep_mem.at(i)->setText(3, QString("R:"));
	    rep_mem.at(i)->setText(4, QString("%1").arg(f_aux));
	    rep_mem.at(i)->setText(5, QString());
	    break;
	  case MEM_UNUSED:
	    rep_mem.at(i)->setText(3, QString());
	    rep_mem.at(i)->setText(4, QString());
	    rep_mem.at(i)->setText(5, QString());
	    break;
	  case MEM_FP:
	    list_mem.at(i)->r_fpo(&i_aux, &i_aux_ref);
	    rep_mem.at(i)->setText(3, QString("FP (%1):").arg(i_aux_ref));
	    rep_mem.at(i)->setText(4, QString("%1").arg(i_aux));
	    rep_mem.at(i)->setText(5, QString());
	    break;
	  case MEM_DEST_S:
	    list_mem.at(i)->r_des(&i_aux, &i_aux_ref);
	    rep_mem.at(i)->setText(3, QString("DIR (%1):").arg(i_aux_ref));
	    rep_mem.at(i)->setText(4, QString("%1").arg(i_aux));
	    rep_mem.at(i)->setText(5, QString());
	    break;
	  default:
	    break;
	} //Fin Switch
    } //Fin For
	    
    if(ult_escr != -1) for(int i = 0; i < 6; i++) rep_mem.at(ult_escr)->setBackground(i, QBrush(Qt::Dense6Pattern));
    
    if(salto != -5) rep_inst.at(inst_actual)->setIcon(0, QIcon(style()->standardPixmap(QStyle::SP_ArrowRight)));
    
    if(desp_mem && ult_escr != -1) tree_mem->scrollToItem(rep_mem.at(ult_escr));
    
    if(desp_mem && salto != -5) tree_inst->scrollToItem(rep_inst.at(inst_actual));
    
    if(salto != -3 || salto != -5){
	accionejecutar->setEnabled(true);
	accionsiguientepaso->setEnabled(true);
	accionparar->setEnabled(true);
	accionejecutar->setEnabled(true);
        accionabrir->setEnabled(true);
        accionrecargar->setEnabled(true);
        accioncompilar->setEnabled(true);
        accioneditar->setEnabled(true);
    }
}
