#include <QtGui>
#include <QProcess>

#include "principal.h"
#include "dialogoopciones.h"


Principal::Principal(){
    
    //Creamos las acciones
    accionabrir = new QAction(tr("&Abrir..."), this);
    accionabrir->setIcon(QIcon(":/iconos/abrir.png"));
    accionabrir->setShortcut(tr("Ctrl+A"));
    accionabrir->setStatusTip(tr("Abre un fichero C3D existente"));
    connect(accionabrir, SIGNAL(triggered()), this, SLOT(abrir()));

    accionrecargar = new QAction(tr("&Recargar"), this);
    accionrecargar->setIcon(QIcon(":/iconos/recargar.png"));
    accionrecargar->setShortcut(tr("Ctrl+R"));
    accionrecargar->setStatusTip(tr("Recarga el fichero C3D actualmente abierto"));
    accionrecargar->setEnabled(false);
    connect(accionrecargar, SIGNAL(triggered()), this, SLOT(recargar()));

    accionejecutar = new QAction(tr("&Ejecutar"), this);
    accionejecutar->setIcon(QIcon(":/iconos/ejecutar.png"));
    accionejecutar->setShortcut(tr("Ctrl+E"));
    accionejecutar->setStatusTip(tr("Ejecuta el fichero C3D desde la posicion actual"));
    accionejecutar->setEnabled(false);
    connect(accionejecutar, SIGNAL(triggered()), this, SLOT(ejecutar()));

    accionsiguientepaso = new QAction(tr("E&jecutar la siguiente instruccion"), this);
    accionsiguientepaso->setIcon(QIcon(":/iconos/paso.png"));
    accionsiguientepaso->setShortcut(tr("Ctrl+J"));
    accionsiguientepaso->setStatusTip(tr("Ejecuta la siguiente instruccion del fichero C3D"));
    accionsiguientepaso->setEnabled(false);
    connect(accionsiguientepaso, SIGNAL(triggered()), this, SLOT(siguientepaso()));

    accionparar = new QAction(tr("&Detiene la ejecucion y reinicia el interprete"), this);
    accionparar->setIcon(QIcon(":/iconos/parar.png"));
    accionparar->setShortcut(tr("Ctrl+D"));
    accionparar->setStatusTip(tr("Detiene la ejecucion del fichero C3D reiniciando su estado"));
    accionparar->setEnabled(false);
    connect(accionparar, SIGNAL(triggered()), this, SLOT(parar()));

    accionborrarbreakpoints = new QAction(tr("&Borrar los breakpoints"), this);
    accionborrarbreakpoints->setStatusTip(tr("Borra los breakpoints que hayas puesto en el programa"));
    accionborrarbreakpoints->setShortcut(tr("Ctrl+B"));
    accionborrarbreakpoints->setEnabled(false);
    connect(accionborrarbreakpoints, SIGNAL(triggered()), this, SLOT(borrarbreakpoints()));

    accionpilaestricta = new QAction(tr("&Memoria estricta"), this);
    accionpilaestricta->setStatusTip(tr("La memoria se debe manejar estrictamente como una pila"));
    accionpilaestricta->setShortcut(tr("Ctrl+I"));
    accionpilaestricta->setCheckable(true);
    accionpilaestricta->setChecked(true);
    connect(accionpilaestricta, SIGNAL(triggered()), this, SLOT(cambiapilaestricta()));
    
    acciondesplazavisor = new QAction(tr("Despla&za la memoria y las instrucciones"), this);
    acciondesplazavisor->setStatusTip(tr("Desplaza la visualizacion de la memoria al ultimo elemento insertado y las instrucciones a la proxima en ejecutarse"));
    acciondesplazavisor->setShortcut(tr("Ctrl+Z"));
    acciondesplazavisor->setCheckable(true);
    acciondesplazavisor->setChecked(true);
    connect(acciondesplazavisor, SIGNAL(triggered()), this, SLOT(desplazavisor()));
 
    accionsalir = new QAction(tr("&Salir"), this);
    accionsalir->setShortcut(tr("Ctrl+S"));
    accionsalir->setStatusTip(tr("Sale de la aplicacion"));
    connect(accionsalir, SIGNAL(triggered()), this, SLOT(close())); //Slot de QMainWindow
    
    accioncompilar = new QAction(tr("Compilar"), this);
    accioncompilar->setStatusTip(tr("Compila el programa cargado y lo recarga"));
    accioncompilar->setIcon(QIcon(":/iconos/compilar.png"));
    accioncompilar->setEnabled(false);
    connect(accioncompilar, SIGNAL(triggered()), this, SLOT(compilar()));
    
    accioneditar = new QAction(tr("Editar"), this);
    accioneditar->setStatusTip(tr("Edita el programa cargado en un editor de texto y lo recarga"));
    accioneditar->setIcon(QIcon(":/iconos/editar.png"));
    accioneditar->setEnabled(false);
    connect(accioneditar, SIGNAL(triggered()), this, SLOT(editar()));

    accionacerca = new QAction(tr("A&cerca de"), this);
    accionacerca->setIcon(QIcon(":/iconos/acerca.png"));
    accionacerca->setShortcut(tr("Ctrl+C"));
    accionacerca->setStatusTip(tr("Muestra la ventana de autoria del programa"));
    connect(accionacerca, SIGNAL(triggered()), this, SLOT(acerca()));

    accionayuda = new QAction(tr("A&yuda"), this);
    accionayuda->setIcon(QIcon(":/iconos/ayuda.png"));
    accionayuda->setShortcut(tr("Ctrl+Y"));
    accionayuda->setStatusTip(tr("Muestra una breve ayuda del programa"));
    connect(accionayuda, SIGNAL(triggered()), this, SLOT(ayuda()));
    
    accionopciones = new QAction(tr("Configuracion"), this);
    accionopciones->setIcon(QIcon(":/iconos/opciones.png"));
    accionopciones->setStatusTip(tr("Permite configurar diversas opciones"));
    connect(accionopciones, SIGNAL(triggered()), this, SLOT(opciones()));
    
    //Creamos el menu de la ventana principal
    menufichero = menuBar()->addMenu(tr("&Archivo"));
    menufichero->addAction(accionabrir);
    menufichero->addAction(accionrecargar);
    menufichero->addAction(accioncompilar);
    menufichero->addAction(accioneditar);
    menufichero->addSeparator();
    menufichero->addAction(accionsalir);

    menuejecutar = menuBar()->addMenu(tr("&Ejecutar"));
    menuejecutar->addAction(accionejecutar);
    menuejecutar->addAction(accionsiguientepaso);
    menuejecutar->addAction(accionparar);

    menuopciones = menuBar()->addMenu(tr("&Opciones"));
    menuopciones->addAction(accionborrarbreakpoints);
    menuopciones->addAction(accionpilaestricta);
    menuopciones->addAction(acciondesplazavisor);
    menuopciones->addAction(accionopciones);

    menuayuda = menuBar()->addMenu(tr("A&yuda"));
    menuayuda->addAction(accionayuda);
    menuayuda->addAction(accionacerca);

    //Creamos las barras de herramientas
    barraarchivo = addToolBar(tr("&Archivo"));
    barraarchivo->addAction(accionabrir);
    barraarchivo->addAction(accionrecargar);
    barraarchivo->addAction(accioncompilar);
    barraarchivo->addAction(accioneditar);

    barraejecutar = addToolBar(tr("&Ejecutar"));
    barraejecutar->addAction(accionejecutar);
    barraejecutar->addAction(accionsiguientepaso);
    barraejecutar->addAction(accionparar);

    barraayuda = addToolBar(tr("A&yuda"));
    barraayuda->addAction(accionayuda);
    barraayuda->addAction(accionacerca);

    //Creamos la barra de estado
    textobarra = new QLabel(" Listo ");
    textobarra->setAlignment(Qt::AlignHCenter);
    textobarra->setMinimumSize(textobarra->sizeHint());
    statusBar()->addWidget(textobarra);
    
    //Establecemos el icono y el titulo de la ventana
    setWindowIcon(QIcon(":/iconos/aplicacion.png"));
    setWindowTitle(tr("%1 - %2").arg("Vacio").arg(tr("Falspas")));
    
    //Establecemos los strings que representaran las instrucciones
    textcodes NOMBRES; //NOMBRES es un define muy 'rudimentario'

    //Creamos el panel que representa las instrucciones
    tree_inst = new QTreeWidget;
    tree_inst->setIndentation(0);
    tree_inst->setHeaderLabels(QStringList() << tr("Ej")<< tr("Bp") << tr("No") << tr("Instruccion") << tr("Arg 1") << tr("Arg 2") << tr("Arg 3"));
    tree_inst->setMouseTracking(true);
    connect(tree_inst, SIGNAL(itemDoubleClicked ( QTreeWidgetItem *, int)), this, SLOT(activarbp(QTreeWidgetItem *, int)));
    
    //Creamos el panel de salida con un color adecuado
    QPalette paletasalida;
    paletasalida.setColor(QPalette::Base, QColor(0, 0, 0, 255));    
    textosalida = new QTextEdit;
    textosalida->setAcceptRichText(false);
    textosalida->setReadOnly(true);
    textosalida->setPalette(paletasalida);
    textosalida->setTextColor(QColor(0, 255, 0, 255));
    
    //Creamos el panel de salida de errores
    textoerrores = new QTextEdit;
    textoerrores->setReadOnly(true);
    textoerrores->setPlainText(QString());
    textoerrores->setTextColor(QColor(255, 0, 0, 255));

    //Creamos el panel de representacion de la memoria
    tree_mem = new QTreeWidget;
    tree_mem->setIndentation(0);
    tree_mem->setHeaderLabels(QStringList() << tr("Pos") << tr("TOP") << tr("FP") << tr("Tipo")<< tr("Valor") << tr("Rep Caracter"));
        
    //Anadimos los items de la memoria vacios
    for(int i=0; i<MAXMEM; i++){
	QTreeWidgetItem *inserta = new QTreeWidgetItem();
	inserta->setText(0, QString("%1").arg(i));
	rep_mem.append(inserta);
    }
    tree_mem->insertTopLevelItems(0, rep_mem);
    
    //Iniciamos la memoria y las instrucciones
    for(int i=0; i<MAXMEM; i++) list_mem.append(new c_tipo_mem());
    for(int i=0; i<MAXCODE; i++) list_inst.append(new c_tipo_ctd());
    
    //Ponemos cada Widget en un lado de la ventana, separandolos con Splitters
    splitterderecho = new QSplitter(Qt::Vertical);
    splitterderecho->addWidget(tree_mem);
    splitterderecho->addWidget(textosalida);
    
    splitterizquierdo = new QSplitter(Qt::Vertical);
    splitterizquierdo->addWidget(tree_inst);
    splitterizquierdo->addWidget(textoerrores);

    splitterprincipal = new QSplitter(Qt::Horizontal);
    splitterprincipal->addWidget(splitterizquierdo);
    splitterprincipal->addWidget(splitterderecho);
    
    setCentralWidget(splitterprincipal);
    
    //Inicializamos ciertas variables y cargamos valores de configuracion
    totalbp = total_inst = inst_actual = 0;
    desp_mem = true;
    muestra_excepciones = true;
    leeconf();
}


// El metodo opciones abre un dialogo donde podemos cambiar la ruta del editor y el compilador.

void Principal::opciones(){
    DialogoOpciones dlg(this, ruta_editor, ruta_compilador);
    if(dlg.exec()){
      switch(dlg.elem_leidos){
        case 1:
          ruta_editor = dlg.ed_editor->text();
          statusBar()->showMessage(tr("La ruta del editor se ha guardado"), 5000); break;
        case 2:
          ruta_compilador = dlg.ed_compilador->text();
          statusBar()->showMessage(tr("La ruta del compilador se ha guardado"), 5000); break;
        case 3:
          ruta_compilador = dlg.ed_compilador->text();
          ruta_editor = dlg.ed_editor->text();
          statusBar()->showMessage(tr("Las opciones se han guardado"), 5000); break;
        default:
          statusBar()->showMessage(tr("Las opciones no se han guardado"), 5000); break;
      }
    }else statusBar()->showMessage(tr("Las opciones no se han guardado"), 5000);
}


// El metodo compilar:
//   1) Toma la ruta completa al programa abierto (variable Principal::ficheroactual)
//   2) Coge el nombre y ruta del fichero y cambia su extension a .c
//   3) Comprueba que ese fichero exista
//   4) Comprueba que el compilador exista
//   5) Ejecuta el compilador pasando como argumento el fichero .c
//   6) Carga de nuevo el programa, suponiendo que la nueva compilacion habra sobreescrito el fichero abierto

void Principal::compilar(){
    QFileInfo info_fichero_actual(ficheroactual);
    //Suponemos que el fichero MenosC esta en el mismo directorio y se llama igual que el fichero cargado pero con extension .c
    QString fichero_a_recompilar = info_fichero_actual.canonicalPath() + "/" + info_fichero_actual.completeBaseName() + ".c";
    QFileInfo info_fichero_c(fichero_a_recompilar);
    if(!info_fichero_c.isReadable()){ //El fichero .c debe tener permisos de lectura
      QMessageBox::warning(this, tr("FalsPas"), tr("Error compilando %1\nEl fichero %2 debe estar en el mismo directorio que %3:\n%4").arg(info_fichero_actual.baseName()).arg(info_fichero_actual.fileName()).arg(info_fichero_c.fileName()).arg(info_fichero_actual.canonicalPath()));
      return;
    }
    if(!QFileInfo(ruta_compilador).isExecutable()){ //El compilador debe existir y tener permisos de ejecucion
      QMessageBox::warning(this, tr("FalsPas"), tr("Error al buscar el compilador en %1\nVerifica la ruta en las opciones y comprueba que tenga permisos de ejecucion.").arg(ruta_compilador));
      return;
    }
    QProcess::execute(ruta_compilador, QStringList() << fichero_a_recompilar); //Suponemos que el compilador no necesita parametros
    cargar(ficheroactual);
}


// El metodo editar:
//   1) Comprueba que el editor existe
//   2) Comprueba que el fichero actual existe y tiene permisos de escritura
//   3) Llama al editor pasando como parametro el actual fichero abierto
//   4) Tras cerrar el editor el fichero se recargara

void Principal::editar(){ //Abre un editor como puede ser el gedit con el fichero que estemos ejecutando, lo modificamos, lo salvamos
    if(!QFileInfo(ruta_editor).isExecutable()){   // y al cerrar el editor se carga automaticamente en la maquina virtual
      QMessageBox::warning(this, tr("FalsPas"), tr("Error al buscar el editor en %1\nVerifica la ruta en las opciones y que se tengan permisos de ejecucion.").arg(ruta_editor));
      return;
    }
    if(!QFileInfo(ficheroactual).isWritable()){
      QMessageBox::warning(this, tr("FalsPas"), tr("Error al intentar editar el fichero %1\nNo se encuentra o no se tienen permisos de escritura").arg(ficheroactual));
      return;
    }  //Suponemos que al editor se llama con el fichero como unico parametro
    QProcess::execute(ruta_editor, QStringList() << ficheroactual);
    cargar(ficheroactual);
}


// El metodo abrir:
//   1) Abre un dialogo para seleccionar un archivo
//   2) En caso de seleccionar un fichero, se cargara usando el metodo cargar

void Principal::abrir(){   
    QString archivoparaabrir = QFileDialog::getOpenFileName(this, tr("Abrir programa"), ".", tr("Codigo 3 direcciones (*.c3d);;Todos los ficheros (*.*)"));
    if (archivoparaabrir.isEmpty()){ //Si la cadena esta vacia significa que hemos pulsado el boton cancelar
	statusBar()->showMessage(tr("Carga cancelada"), 5000);
    } else cargar(archivoparaabrir);    
}


// El metodo activarbp se llama con la señal emitida al hacer doble clic sobre una instruccion.
// Si se hace clic en una instruccion con breakpoint, el breakpoint se elimina.
// Si la instruccion no tiene breakpoint, se anyade.

void Principal::activarbp(QTreeWidgetItem * item, int column){ //Column es un variable que se usa como un auxiliar. Mala idea por no declarar un entero.
    column = item->text(2).toInt(); //Tenemos un puntero al item en vez de un numero de instruccion. Al menos podemos leer la cadena de texto.
    if(list_inst.at(column)->r_brp()){
	list_inst.at(column)->s_brp(false);
	item->setIcon(1, QIcon());
	totalbp--;
	if(totalbp <= 0) accionborrarbreakpoints->setEnabled(false);
	statusBar()->showMessage(tr("Desactivado BP en linea %1").arg(column), 2000);
    }else{
	list_inst.at(column)->s_brp(true);
	item->setIcon(1, QIcon(style()->standardPixmap(QStyle::SP_MessageBoxCritical)));
	accionborrarbreakpoints->setEnabled(true);
	totalbp++;
	statusBar()->showMessage(tr("Activado BP en linea %1").arg(column), 2000);
    }
}


// El metodo acerca muestra una ventana con la autoria del proyecto

void Principal::acerca(){ //Pequeña caja de texto con una descripcion del proyecto
    QMessageBox::about(this, tr("Acerca de Falspas"), tr("<h2>Fals Pas</h2><p>Int&eacute;rprete gr&aacute;fico f&aacute;cil de usar, ligero y sencillo<p>Falspas interpreta el c&oacute;digo de tres direcciones en un entorno visual de depuraci&oacute;n intuitivo.<p>PFC Alberto Dominguez, 2010"));
}


// El metodo recargar vuelve a cargar el fichero abierto

void Principal::recargar(){
    cargar(ficheroactual);  //Es el metodo cargar el que comprueba que el fichero exista, los permisos, etc.
}


// El metodo parar:
//  1) Inicializa las variables Principal::inst_actual, Principal::p_top, Principal::p_fp y Principal::ult_escr.
//  2) Borramos la posible flecha indicadora del contador de programa
//  3) Borramos la memoria, su representacion y el indicador de ultimo dato escrito
//  4) Reseteamos las indicaciones de direcciones fisicas
//  5) Representamos graficamente los nuevos punteros a cima y marco

void Principal::parar(){ // Debemos borrar la memoria, poner a cero el p_fp, p_top e inst_actual y hacer que la memoria y las instrucciones se vean donde toca
    int i, j;
    inst_actual = p_top = p_fp = 0;
    ult_escr = -1; //Ponemos a cero (o equivalente) ciertas variables de estado
    for(i = 0; i < total_inst; i++) rep_inst.at(i)->setIcon(0, QIcon());               //Si hemos llegado al estado de fallo por error en PC
    rep_inst.at(0)->setIcon(0, QIcon(style()->standardPixmap(QStyle::SP_ArrowRight))); //no sabremos REALMENTE cual es la posicion de la flecha
    for(i = 0; i < MAXMEM; i++){ //Tampoco podemos dar por hecho que la memoria por encima del TOP estara vacia, puesto que podemos ejecutar
	rep_mem.at(i)->setIcon(1, QIcon()); //el programa sin el modo estricto
	rep_mem.at(i)->setIcon(2, QIcon());
	if(list_mem.at(i)->r_tip() != MEM_UNUSED){
	    list_mem.at(i)->reset();
	    rep_mem.at(i)->setText(3, QString());
	    rep_mem.at(i)->setText(4, QString());
	    rep_mem.at(i)->setText(5, QString());
	}
	for(j = 0; j < 6; j++) rep_mem.at(i)->setBackground(j, QBrush(Qt::NoBrush)); //Esta quizas se pudiese sacar del bucle
    }
    set_dir_fisicas();
    rep_mem.at(0)->setIcon(1, QIcon(":/iconos/flecha_azul.png"));
    rep_mem.at(0)->setIcon(2, QIcon(":/iconos/flecha_verde.png"));
    accionejecutar->setEnabled(true);
    accionsiguientepaso->setEnabled(true);
    accionparar->setEnabled(false);
    textosalida->setText(QString());
    textoerrores->setText(QString());
    statusBar()->showMessage(tr("Se ha parado su programa y reiniciado el interprete"), 5000);
}


// El metodo borrarbreakpoints borra todos los breakpoints que existan en el programa

void Principal::borrarbreakpoints(){ //Borra los breakpoints. En el caso de no haber la accion no deberia estar activa
    int i, cont_borrados = 0;
    accionborrarbreakpoints->setEnabled(false);
    totalbp = 0;
    for(i = 0; i <= total_inst; i++) if(list_inst.at(i)->r_brp()){
      list_inst.at(i)->s_brp(false);
      rep_inst.at(i)->setIcon(1, QIcon());
      cont_borrados++;
    }
    if(cont_borrados == 1) statusBar()->showMessage(tr("Se ha borrado el breakpoint activo"), 5000);
    else statusBar()->showMessage(tr("Se han borrado los %1 breakpoints activos").arg(cont_borrados), 5000);
}


// El metodo cambiapilaestricta solo cambia el estado de la variable bool Principal::pila_estricta

void Principal::cambiapilaestricta(){ //Cambia el estado de la variable pila_estricta. Modifica los permisos para acceder a la memoria.
    pila_estricta = !pila_estricta; //Si pila_estricta == true, la memoria por encima de TOP no sera accesible en ningun concepto.
    if(pila_estricta) statusBar()->showMessage(tr("La pila de memoria debe usarse de forma estricta"), 5000);
    else statusBar()->showMessage(tr("Se puede acceder a la memoria con total libertad"), 5000);
}

void Principal::ayuda(){ //Aqui deberemos mostrar un cuadro con pestañas indicando algunos conceptos de la maquina virtual
    QMessageBox::about(this, QString(tr("Ayuda de FalsPas")), QString(tr("Para establecer un breakpoint haga doble clic sobre una instrucci&oacute;n. Para eliminarlo vuelva a hacer doble clic.<p>La opci&oacute;n <b>Memoria estricta</b> provoca que cualquier dato que quede por encima del puntero a pila sea inaccesible. No se podr&aacute; leer ni escribir, y al modificar el puntero a pila cualquier dato que quede en posiciones de memoria m&aacute;s altas se borrar&aacute;n. Adem&aacute;s la instrucci&oacute;n <b>RET</b> y <b>FPPOP</b> solo admitir&aacute;n enteros que hayan sido escritos en memoria por las instrucciones <b>CALL</b> y <b>PUSHFP</b> respectivamente.<p>Tambi&eacute;n puede salir de un programa ejecut&aacute;ndose en modo continuo si hace clic en <b>cancelar</b> en un di&aacute;logo de petici&oacute;n de datos.")));
}


// El metodo desplazavisor solo cambia el estado de la variable bool Principal::desp_mem

void Principal::desplazavisor(){ //Cambia el estado de la variable desp_mem.
    desp_mem = !desp_mem;        //Si desp_mem == true, el panel de memoria hara scroll a la posicion que se escriba.
    if(desp_mem) statusBar()->showMessage(tr("La lista de instrucciones y la memoria tendran scroll automatico"), 5000);
    else statusBar()->showMessage(tr("La lista de instrucciones y la memoria no tendran scroll automatico"), 5000);
}


// Escribeconf escribe la configuracion de la maquina virtual al cerrar el programa

void Principal::escribeconf(){ //Escribe la configuracion que se cargara la proxima vez que se abra el programa
  QSettings conf("aldolo", "falspas");
  conf.setValue("geometria", geometry()); //Posicion de las cuatro esquinas de la ventana
  conf.setValue("splitterprincipal", splitterprincipal->saveState());
  conf.setValue("splitterderecho", splitterderecho->saveState());
  conf.setValue("splitterizquierdo", splitterizquierdo->saveState());
  conf.setValue("compilador", ruta_compilador);
  conf.setValue("editor", ruta_editor);
  conf.setValue("pila", pila_estricta);
  conf.setValue("desplazarvisor", desp_mem);
  conf.setValue("col0inst", tree_inst->columnWidth(0)); 
  conf.setValue("col1inst", tree_inst->columnWidth(1)); 
  conf.setValue("col2inst", tree_inst->columnWidth(2)); 
  conf.setValue("col3inst", tree_inst->columnWidth(3)); 
  conf.setValue("col4inst", tree_inst->columnWidth(4)); 
  conf.setValue("col5inst", tree_inst->columnWidth(5)); 
  conf.setValue("col6inst", tree_inst->columnWidth(6));
  conf.setValue("col0mem", tree_mem->columnWidth(0)); 
  conf.setValue("col1mem", tree_mem->columnWidth(1)); 
  conf.setValue("col2mem", tree_mem->columnWidth(2)); 
  conf.setValue("col3mem", tree_mem->columnWidth(3)); 
  conf.setValue("col4mem", tree_mem->columnWidth(4)); 
  conf.setValue("col5mem", tree_mem->columnWidth(5)); 
}


// Leeconf lee la configuracion de la maquina virtual al iniciarse

void Principal::leeconf(){ //Lee y carga la configuracion cuando se abre el programa
  QSettings conf("aldolo", "falspas");
  QRect rectangulo = conf.value("geometria", QRect(200, 200, 400, 400)).toRect();
  move(rectangulo.topLeft());
  resize(rectangulo.size());
  splitterprincipal->restoreState(conf.value("splitterprincipal").toByteArray());
  splitterderecho->restoreState(conf.value("splittersecundario").toByteArray());
  splitterizquierdo->restoreState(conf.value("splitterizquierdo").toByteArray());
  accionpilaestricta->setChecked(pila_estricta = conf.value("pila", true).toBool());
  acciondesplazavisor->setChecked(desp_mem = conf.value("desplazarvisor", true).toBool());
  ruta_compilador = conf.value("compilador", QString("./cmc")).toString();
  ruta_editor = conf.value("editor", QString("/usr/bin/gedit")).toString();
  tree_inst->setColumnWidth(0, conf.value("col0inst", 30).toInt()); 
  tree_inst->setColumnWidth(1, conf.value("col1inst", 30).toInt()); 
  tree_inst->setColumnWidth(2, conf.value("col2inst", 30).toInt()); 
  tree_inst->setColumnWidth(3, conf.value("col3inst", 30).toInt()); 
  tree_inst->setColumnWidth(4, conf.value("col4inst", 30).toInt()); 
  tree_inst->setColumnWidth(5, conf.value("col5inst", 30).toInt()); 
  tree_inst->setColumnWidth(6, conf.value("col6inst", 30).toInt());
  tree_mem->setColumnWidth(0, conf.value("col0mem", 30).toInt()); 
  tree_mem->setColumnWidth(1, conf.value("col1mem", 30).toInt()); 
  tree_mem->setColumnWidth(2, conf.value("col2mem", 30).toInt()); 
  tree_mem->setColumnWidth(3, conf.value("col3mem", 30).toInt()); 
  tree_mem->setColumnWidth(4, conf.value("col4mem", 30).toInt()); 
  tree_mem->setColumnWidth(5, conf.value("col5mem", 30).toInt()); 
}


// Reescribimos el metodo closeEvent para que la maquina virtual guarde su configuracion cuando se cierre

void Principal::closeEvent(QCloseEvent *event){ //Evento sobrecargado para que cuando se cierre la maquina virtual se
    escribeconf();                              //escriba la configuracion
    event->accept();
}
